@@ -231,6 +231,8 @@ typedef struct av1_codec_arg_definitions {
   arg_def_t second_pass_log;
   arg_def_t auto_intra_tools_off;
   arg_def_t strict_level_conformance;
+  arg_def_t kf_max_pyr_height;
+  arg_def_t sb_qp_sweep;
 #endif  // CONFIG_AV1_ENCODER
 } av1_codec_arg_definitions_t;
 
