@@ -102,23 +102,33 @@ static INLINE int av1_superres_scaled(const AV1_COMMON *cm) {
   // Note: for some corner cases (e.g. cm->width of 1), there may be no scaling
   // required even though cm->superres_scale_denominator != SCALE_NUMERATOR.
   // So, the following check is more accurate.
-  return !(cm->width == cm->superres_upscaled_width);
+  return (cm->width != cm->superres_upscaled_width);
 }
 
-// There's SIMD optimizations for 1/4, 1/2 and 3/4 downscaling.
-// SSSE3 also has optimizations for 2x upscaling.
-// Use non normative scalers for other scaling ratios.
+// The optimized scaler av1_resize_and_extend_frame() can only handle scaling
+// ratios >= 1/4 and <= 16. See comment in aom_convolve8_c() for detail.
+// Visual assessment shows that if the scaling ratio or its reciprocal is not a
+// multiple of 1/16, there are some artifacts in the output of the optimized
+// scaler, especially on lines, due to non-exact ratio representation. SSSE3
+// and NEON have a specialized 3/4 version of av1_resize_and_extend_frame()
+// that does not have this issue.
+//
+// Use the non-normative scaler av1_resize_and_extend_frame_nonnormative()
+// for other scaling ratios.
 static INLINE bool av1_has_optimized_scaler(const int src_width,
                                             const int src_height,
                                             const int dst_width,
                                             const int dst_height) {
-  const bool has_optimized_scaler =
-      (dst_width * 4 == src_width && dst_height * 4 == src_height) ||
-      (dst_width * 2 == src_width && dst_height * 2 == src_height) ||
-      (dst_width * 4 == src_width * 3 && dst_height * 4 == src_height * 3);
-#if HAVE_SSSE3
-  return has_optimized_scaler ||
-         (dst_width == src_width * 2 && dst_height == src_height * 2);
+  bool has_optimized_scaler =
+      (dst_width * 4 >= src_width && dst_height * 4 >= src_height) &&
+      (dst_width <= src_width * 16 && dst_height <= src_height * 16) &&
+      (16 * dst_width % src_width == 0) && (16 * src_width % dst_width == 0) &&
+      (16 * dst_height % src_height == 0) &&
+      (16 * src_height % dst_height == 0);
+#if HAVE_SSSE3 || HAVE_NEON
+  has_optimized_scaler =
+      has_optimized_scaler ||
+      (4 * dst_width == 3 * src_width && 4 * dst_height == 3 * src_height);
 #endif
   return has_optimized_scaler;
 }
