@@ -9,6 +9,7 @@
  * PATENTS file, you can obtain it at www.aomedia.org/license/patent.
  */
 
+#include "aom/aom_image.h"
 #include "config/aom_config.h"
 #include "config/aom_scale_rtcd.h"
 
@@ -18,6 +19,7 @@
 #include "av1/common/entropymode.h"
 #include "av1/common/thread_common.h"
 #include "av1/common/reconinter.h"
+#include "av1/common/reconintra.h"
 
 // Set up nsync by width.
 static INLINE int get_sync_range(int width) {
@@ -33,7 +35,6 @@ static INLINE int get_sync_range(int width) {
     return 8;
 }
 
-#if !CONFIG_REALTIME_ONLY
 static INLINE int get_lr_sync_range(int width) {
 #if 0
   // nsync numbers are picked by testing. For example, for 4k
@@ -51,7 +52,6 @@ static INLINE int get_lr_sync_range(int width) {
   return 1;
 #endif
 }
-#endif
 
 // Allocate memory for lf row synchronization
 void av1_loop_filter_alloc(AV1LfSync *lf_sync, AV1_COMMON *cm, int rows,
@@ -138,20 +138,6 @@ void av1_loop_filter_dealloc(AV1LfSync *lf_sync) {
   }
 }
 
-static void loop_filter_data_reset(LFWorkerData *lf_data,
-                                   YV12_BUFFER_CONFIG *frame_buffer,
-                                   struct AV1Common *cm, MACROBLOCKD *xd) {
-  struct macroblockd_plane *pd = xd->plane;
-  lf_data->frame_buffer = frame_buffer;
-  lf_data->cm = cm;
-  lf_data->xd = xd;
-  for (int i = 0; i < MAX_MB_PLANE; i++) {
-    memcpy(&lf_data->planes[i].dst, &pd[i].dst, sizeof(lf_data->planes[i].dst));
-    lf_data->planes[i].subsampling_x = pd[i].subsampling_x;
-    lf_data->planes[i].subsampling_y = pd[i].subsampling_y;
-  }
-}
-
 void av1_alloc_cdef_sync(AV1_COMMON *const cm, AV1CdefSync *cdef_sync,
                          int num_workers) {
   if (num_workers < 1) return;
@@ -261,82 +247,45 @@ static INLINE void sync_write(AV1LfSync *const lf_sync, int r, int c,
 #endif  // CONFIG_MULTITHREAD
 }
 
-static void enqueue_lf_jobs(AV1LfSync *lf_sync, int start, int stop,
-                            const int planes_to_lf[3], int is_realtime) {
-  int mi_row, plane, dir;
-  AV1LfMTInfo *lf_job_queue = lf_sync->job_queue;
-  lf_sync->jobs_enqueued = 0;
-  lf_sync->jobs_dequeued = 0;
-
-  // Launch all vertical jobs first, as they are blocking the horizontal ones.
-  // Launch top row jobs for all planes first, in case the output can be
-  // partially reconstructed row by row.
-  for (dir = 0; dir < 2; ++dir) {
-    for (mi_row = start; mi_row < stop; mi_row += MAX_MIB_SIZE) {
-      for (plane = 0; plane < 3; ++plane) {
-        if (!planes_to_lf[plane]) continue;
-        lf_job_queue->mi_row = mi_row;
-        lf_job_queue->plane = plane;
-        lf_job_queue->dir = dir;
-        lf_job_queue->is_realtime = is_realtime;
-        lf_job_queue++;
-        lf_sync->jobs_enqueued++;
-      }
-    }
-  }
-}
-
-static AV1LfMTInfo *get_lf_job_info(AV1LfSync *lf_sync) {
-  AV1LfMTInfo *cur_job_info = NULL;
-
-#if CONFIG_MULTITHREAD
-  pthread_mutex_lock(lf_sync->job_mutex);
-
-  if (lf_sync->jobs_dequeued < lf_sync->jobs_enqueued) {
-    cur_job_info = lf_sync->job_queue + lf_sync->jobs_dequeued;
-    lf_sync->jobs_dequeued++;
-  }
-
-  pthread_mutex_unlock(lf_sync->job_mutex);
-#else
-  (void)lf_sync;
-#endif
-
-  return cur_job_info;
-}
-
 // One job of row loopfiltering.
-static INLINE void thread_loop_filter_rows(
+void av1_thread_loop_filter_rows(
     const YV12_BUFFER_CONFIG *const frame_buffer, AV1_COMMON *const cm,
     struct macroblockd_plane *planes, MACROBLOCKD *xd, int mi_row, int plane,
-    int dir, int is_realtime, AV1LfSync *const lf_sync) {
+    int dir, int lpf_opt_level, AV1LfSync *const lf_sync,
+    AV1_DEBLOCKING_PARAMETERS *params_buf, TX_SIZE *tx_buf,
+    int num_mis_in_lpf_unit_height_log2) {
   const int sb_cols =
-      ALIGN_POWER_OF_TWO(cm->mi_params.mi_cols, MAX_MIB_SIZE_LOG2) >>
-      MAX_MIB_SIZE_LOG2;
-  const int r = mi_row >> MAX_MIB_SIZE_LOG2;
+      CEIL_POWER_OF_TWO(cm->mi_params.mi_cols, MAX_MIB_SIZE_LOG2);
+  const int r = mi_row >> num_mis_in_lpf_unit_height_log2;
   int mi_col, c;
 
+  const bool joint_filter_chroma = (lpf_opt_level == 2) && plane > AOM_PLANE_Y;
+  const int num_planes = joint_filter_chroma ? 2 : 1;
+  assert(IMPLIES(joint_filter_chroma, plane == AOM_PLANE_U));
+
   if (dir == 0) {
     for (mi_col = 0; mi_col < cm->mi_params.mi_cols; mi_col += MAX_MIB_SIZE) {
       c = mi_col >> MAX_MIB_SIZE_LOG2;
 
       av1_setup_dst_planes(planes, cm->seq_params->sb_size, frame_buffer,
-                           mi_row, mi_col, plane, plane + 1);
-#if CONFIG_AV1_HIGHBITDEPTH
-      (void)is_realtime;
-      av1_filter_block_plane_vert(cm, xd, plane, &planes[plane], mi_row,
-                                  mi_col);
-#else
-      if (is_realtime) {
-        av1_filter_block_plane_vert_rt(cm, xd, plane, &planes[plane], mi_row,
-                                       mi_col);
-
+                           mi_row, mi_col, plane, plane + num_planes);
+      if (lpf_opt_level) {
+        if (plane == AOM_PLANE_Y) {
+          av1_filter_block_plane_vert_opt(cm, xd, &planes[plane], mi_row,
+                                          mi_col, params_buf, tx_buf,
+                                          num_mis_in_lpf_unit_height_log2);
+        } else {
+          av1_filter_block_plane_vert_opt_chroma(
+              cm, xd, &planes[plane], mi_row, mi_col, params_buf, tx_buf, plane,
+              joint_filter_chroma, num_mis_in_lpf_unit_height_log2);
+        }
       } else {
         av1_filter_block_plane_vert(cm, xd, plane, &planes[plane], mi_row,
                                     mi_col);
       }
-#endif
-      if (lf_sync != NULL) sync_write(lf_sync, r, c, sb_cols, plane);
+      if (lf_sync != NULL) {
+        sync_write(lf_sync, r, c, sb_cols, plane);
+      }
     }
   } else if (dir == 1) {
     for (mi_col = 0; mi_col < cm->mi_params.mi_cols; mi_col += MAX_MIB_SIZE) {
@@ -352,20 +301,21 @@ static INLINE void thread_loop_filter_rows(
       }
 
       av1_setup_dst_planes(planes, cm->seq_params->sb_size, frame_buffer,
-                           mi_row, mi_col, plane, plane + 1);
-#if CONFIG_AV1_HIGHBITDEPTH
-      (void)is_realtime;
-      av1_filter_block_plane_horz(cm, xd, plane, &planes[plane], mi_row,
-                                  mi_col);
-#else
-      if (is_realtime) {
-        av1_filter_block_plane_horz_rt(cm, xd, plane, &planes[plane], mi_row,
-                                       mi_col);
+                           mi_row, mi_col, plane, plane + num_planes);
+      if (lpf_opt_level) {
+        if (plane == AOM_PLANE_Y) {
+          av1_filter_block_plane_horz_opt(cm, xd, &planes[plane], mi_row,
+                                          mi_col, params_buf, tx_buf,
+                                          num_mis_in_lpf_unit_height_log2);
+        } else {
+          av1_filter_block_plane_horz_opt_chroma(
+              cm, xd, &planes[plane], mi_row, mi_col, params_buf, tx_buf, plane,
+              joint_filter_chroma, num_mis_in_lpf_unit_height_log2);
+        }
       } else {
         av1_filter_block_plane_horz(cm, xd, plane, &planes[plane], mi_row,
                                     mi_col);
       }
-#endif
     }
   }
 }
@@ -376,11 +326,12 @@ static int loop_filter_row_worker(void *arg1, void *arg2) {
   LFWorkerData *const lf_data = (LFWorkerData *)arg2;
   AV1LfMTInfo *cur_job_info;
   while ((cur_job_info = get_lf_job_info(lf_sync)) != NULL) {
-    const int is_realtime = cur_job_info->is_realtime && !cur_job_info->plane;
-    thread_loop_filter_rows(lf_data->frame_buffer, lf_data->cm, lf_data->planes,
-                            lf_data->xd, cur_job_info->mi_row,
-                            cur_job_info->plane, cur_job_info->dir, is_realtime,
-                            lf_sync);
+    const int lpf_opt_level = cur_job_info->lpf_opt_level;
+    av1_thread_loop_filter_rows(
+        lf_data->frame_buffer, lf_data->cm, lf_data->planes, lf_data->xd,
+        cur_job_info->mi_row, cur_job_info->plane, cur_job_info->dir,
+        lpf_opt_level, lf_sync, lf_data->params_buf, lf_data->tx_buf,
+        MAX_MIB_SIZE_LOG2);
   }
   return 1;
 }
@@ -389,27 +340,11 @@ static void loop_filter_rows_mt(YV12_BUFFER_CONFIG *frame, AV1_COMMON *cm,
                                 MACROBLOCKD *xd, int start, int stop,
                                 const int planes_to_lf[3], AVxWorker *workers,
                                 int num_workers, AV1LfSync *lf_sync,
-                                int is_realtime) {
+                                int lpf_opt_level) {
   const AVxWorkerInterface *const winterface = aom_get_worker_interface();
-  // Number of superblock rows and cols
-  const int sb_rows =
-      ALIGN_POWER_OF_TWO(cm->mi_params.mi_rows, MAX_MIB_SIZE_LOG2) >>
-      MAX_MIB_SIZE_LOG2;
   int i;
-
-  if (!lf_sync->sync_range || sb_rows != lf_sync->rows ||
-      num_workers > lf_sync->num_workers) {
-    av1_loop_filter_dealloc(lf_sync);
-    av1_loop_filter_alloc(lf_sync, cm, sb_rows, cm->width, num_workers);
-  }
-
-  // Initialize cur_sb_col to -1 for all SB rows.
-  for (i = 0; i < MAX_MB_PLANE; i++) {
-    memset(lf_sync->cur_sb_col[i], -1,
-           sizeof(*(lf_sync->cur_sb_col[i])) * sb_rows);
-  }
-
-  enqueue_lf_jobs(lf_sync, start, stop, planes_to_lf, is_realtime);
+  loop_filter_frame_mt_init(cm, start, stop, planes_to_lf, num_workers, lf_sync,
+                            lpf_opt_level, MAX_MIB_SIZE_LOG2);
 
   // Set up loopfilter thread data.
   for (i = num_workers - 1; i >= 0; --i) {
@@ -439,16 +374,23 @@ static void loop_filter_rows_mt(YV12_BUFFER_CONFIG *frame, AV1_COMMON *cm,
 
 static void loop_filter_rows(YV12_BUFFER_CONFIG *frame, AV1_COMMON *cm,
                              MACROBLOCKD *xd, int start, int stop,
-                             const int planes_to_lf[3], int is_realtime) {
+                             const int planes_to_lf[3], int lpf_opt_level) {
   // Filter top rows of all planes first, in case the output can be partially
   // reconstructed row by row.
   int mi_row, plane, dir;
+
+  AV1_DEBLOCKING_PARAMETERS params_buf[MAX_MIB_SIZE];
+  TX_SIZE tx_buf[MAX_MIB_SIZE];
   for (mi_row = start; mi_row < stop; mi_row += MAX_MIB_SIZE) {
     for (plane = 0; plane < 3; ++plane) {
-      if (!planes_to_lf[plane]) continue;
+      if (skip_loop_filter_plane(planes_to_lf, plane, lpf_opt_level)) {
+        continue;
+      }
+
       for (dir = 0; dir < 2; ++dir) {
-        thread_loop_filter_rows(frame, cm, xd->plane, xd, mi_row, plane, dir,
-                                is_realtime && !plane, /*lf_sync=*/NULL);
+        av1_thread_loop_filter_rows(frame, cm, xd->plane, xd, mi_row, plane,
+                                    dir, lpf_opt_level, /*lf_sync=*/NULL,
+                                    params_buf, tx_buf, MAX_MIB_SIZE_LOG2);
       }
     }
   }
@@ -458,19 +400,13 @@ void av1_loop_filter_frame_mt(YV12_BUFFER_CONFIG *frame, AV1_COMMON *cm,
                               MACROBLOCKD *xd, int plane_start, int plane_end,
                               int partial_frame, AVxWorker *workers,
                               int num_workers, AV1LfSync *lf_sync,
-                              int is_realtime) {
+                              int lpf_opt_level) {
   int start_mi_row, end_mi_row, mi_rows_to_filter;
   int planes_to_lf[3];
 
-  // For each luma and chroma plane, whether to filter it or not.
-  planes_to_lf[0] = (cm->lf.filter_level[0] || cm->lf.filter_level[1]) &&
-                    plane_start <= 0 && 0 < plane_end;
-  planes_to_lf[1] = cm->lf.filter_level_u && plane_start <= 1 && 1 < plane_end;
-  planes_to_lf[2] = cm->lf.filter_level_v && plane_start <= 2 && 2 < plane_end;
-  // If the luma plane is purposely not filtered, neither are the chroma planes.
-  if (!planes_to_lf[0] && plane_start <= 0 && 0 < plane_end) return;
-  // Early exit.
-  if (!planes_to_lf[0] && !planes_to_lf[1] && !planes_to_lf[2]) return;
+  if (!check_planes_to_loop_filter(&cm->lf, planes_to_lf, plane_start,
+                                   plane_end))
+    return;
 
   start_mi_row = 0;
   mi_rows_to_filter = cm->mi_params.mi_rows;
@@ -485,15 +421,14 @@ void av1_loop_filter_frame_mt(YV12_BUFFER_CONFIG *frame, AV1_COMMON *cm,
   if (num_workers > 1) {
     // Enqueue and execute loopfiltering jobs.
     loop_filter_rows_mt(frame, cm, xd, start_mi_row, end_mi_row, planes_to_lf,
-                        workers, num_workers, lf_sync, is_realtime);
+                        workers, num_workers, lf_sync, lpf_opt_level);
   } else {
     // Directly filter in the main thread.
     loop_filter_rows(frame, cm, xd, start_mi_row, end_mi_row, planes_to_lf,
-                     is_realtime);
+                     lpf_opt_level);
   }
 }
 
-#if !CONFIG_REALTIME_ONLY
 static INLINE void lr_sync_read(void *const lr_sync, int r, int c, int plane) {
 #if CONFIG_MULTITHREAD
   AV1LrSync *const loop_res_sync = (AV1LrSync *)lr_sync;
@@ -685,7 +620,7 @@ static void enqueue_lr_jobs(AV1LrSync *lr_sync, AV1LrStruct *lr_ctxt,
     const int is_uv = plane > 0;
     const int ss_y = is_uv && cm->seq_params->subsampling_y;
 
-    AV1PixelRect tile_rect = ctxt[plane].tile_rect;
+    PixelRect tile_rect = ctxt[plane].tile_rect;
     const int unit_size = ctxt[plane].rsi->restoration_unit_size;
 
     const int tile_h = tile_rect.bottom - tile_rect.top;
@@ -808,6 +743,12 @@ static int loop_restoration_row_worker(void *arg1, void *arg2) {
       copy_funs[plane](lr_ctxt->dst, lr_ctxt->frame, ctxt[plane].tile_rect.left,
                        ctxt[plane].tile_rect.right, cur_job_info->v_copy_start,
                        cur_job_info->v_copy_end);
+
+      if (lrworkerdata->do_extend_border) {
+        aom_extend_frame_borders_plane_row(lr_ctxt->frame, plane,
+                                           cur_job_info->v_copy_start,
+                                           cur_job_info->v_copy_end);
+      }
     } else {
       break;
     }
@@ -817,7 +758,8 @@ static int loop_restoration_row_worker(void *arg1, void *arg2) {
 
 static void foreach_rest_unit_in_planes_mt(AV1LrStruct *lr_ctxt,
                                            AVxWorker *workers, int nworkers,
-                                           AV1LrSync *lr_sync, AV1_COMMON *cm) {
+                                           AV1LrSync *lr_sync, AV1_COMMON *cm,
+                                           int do_extend_border) {
   FilterFrameCtxt *ctxt = lr_ctxt->ctxt;
 
   const int num_planes = av1_num_planes(cm);
@@ -828,7 +770,7 @@ static void foreach_rest_unit_in_planes_mt(AV1LrStruct *lr_ctxt,
   for (int plane = 0; plane < num_planes; plane++) {
     if (cm->rst_info[plane].frame_restoration_type == RESTORE_NONE) continue;
 
-    const AV1PixelRect tile_rect = ctxt[plane].tile_rect;
+    const PixelRect tile_rect = ctxt[plane].tile_rect;
     const int max_tile_h = tile_rect.bottom - tile_rect.top;
 
     const int unit_size = cm->rst_info[plane].restoration_unit_size;
@@ -860,6 +802,7 @@ static void foreach_rest_unit_in_planes_mt(AV1LrStruct *lr_ctxt,
   for (i = num_workers - 1; i >= 0; --i) {
     AVxWorker *const worker = &workers[i];
     lr_sync->lrworkerdata[i].lr_ctxt = (void *)lr_ctxt;
+    lr_sync->lrworkerdata[i].do_extend_border = do_extend_border;
     worker->hook = loop_restoration_row_worker;
     worker->data1 = lr_sync;
     worker->data2 = &lr_sync->lrworkerdata[i];
@@ -881,7 +824,8 @@ static void foreach_rest_unit_in_planes_mt(AV1LrStruct *lr_ctxt,
 void av1_loop_restoration_filter_frame_mt(YV12_BUFFER_CONFIG *frame,
                                           AV1_COMMON *cm, int optimized_lr,
                                           AVxWorker *workers, int num_workers,
-                                          AV1LrSync *lr_sync, void *lr_ctxt) {
+                                          AV1LrSync *lr_sync, void *lr_ctxt,
+                                          int do_extend_border) {
   assert(!cm->features.all_lossless);
 
   const int num_planes = av1_num_planes(cm);
@@ -892,9 +836,8 @@ void av1_loop_restoration_filter_frame_mt(YV12_BUFFER_CONFIG *frame,
                                          optimized_lr, num_planes);
 
   foreach_rest_unit_in_planes_mt(loop_rest_ctxt, workers, num_workers, lr_sync,
-                                 cm);
+                                 cm, do_extend_border);
 }
-#endif
 
 // Initializes cdef_sync parameters.
 static AOM_INLINE void reset_cdef_job_info(AV1CdefSync *const cdef_sync) {
@@ -966,13 +909,27 @@ static AOM_INLINE int get_cdef_row_next_job(AV1CdefSync *const cdef_sync,
 static int cdef_sb_row_worker_hook(void *arg1, void *arg2) {
   AV1CdefSync *const cdef_sync = (AV1CdefSync *)arg1;
   AV1CdefWorkerData *const cdef_worker = (AV1CdefWorkerData *)arg2;
-  const int nvfb =
-      (cdef_worker->cm->mi_params.mi_rows + MI_SIZE_64X64 - 1) / MI_SIZE_64X64;
+  AV1_COMMON *cm = cdef_worker->cm;
+  const int nvfb = (cm->mi_params.mi_rows + MI_SIZE_64X64 - 1) / MI_SIZE_64X64;
   int cur_fbr;
+  const int num_planes = av1_num_planes(cm);
   while (get_cdef_row_next_job(cdef_sync, &cur_fbr, nvfb)) {
-    av1_cdef_fb_row(cdef_worker->cm, cdef_worker->xd, cdef_worker->linebuf,
-                    cdef_worker->colbuf, cdef_worker->srcbuf, cur_fbr,
+    MACROBLOCKD *xd = cdef_worker->xd;
+    av1_cdef_fb_row(cm, xd, cdef_worker->linebuf, cdef_worker->colbuf,
+                    cdef_worker->srcbuf, cur_fbr,
                     cdef_worker->cdef_init_fb_row_fn, cdef_sync);
+    if (cdef_worker->do_extend_border) {
+      for (int plane = 0; plane < num_planes; ++plane) {
+        const YV12_BUFFER_CONFIG *ybf = &cm->cur_frame->buf;
+        const int is_uv = plane > 0;
+        const int mi_high = MI_SIZE_LOG2 - xd->plane[plane].subsampling_y;
+        const int unit_height = MI_SIZE_64X64 << mi_high;
+        const int v_start = cur_fbr * unit_height;
+        const int v_end =
+            AOMMIN(v_start + unit_height, ybf->crop_heights[is_uv]);
+        aom_extend_frame_borders_plane_row(ybf, plane, v_start, v_end);
+      }
+    }
   }
   return 1;
 }
@@ -981,7 +938,8 @@ static int cdef_sb_row_worker_hook(void *arg1, void *arg2) {
 static void prepare_cdef_frame_workers(
     AV1_COMMON *const cm, MACROBLOCKD *xd, AV1CdefWorkerData *const cdef_worker,
     AVxWorkerHook hook, AVxWorker *const workers, AV1CdefSync *const cdef_sync,
-    int num_workers, cdef_init_fb_row_t cdef_init_fb_row_fn) {
+    int num_workers, cdef_init_fb_row_t cdef_init_fb_row_fn,
+    int do_extend_border) {
   const int num_planes = av1_num_planes(cm);
 
   cdef_worker[0].srcbuf = cm->cdef_info.srcbuf;
@@ -992,6 +950,7 @@ static void prepare_cdef_frame_workers(
     cdef_worker[i].cm = cm;
     cdef_worker[i].xd = xd;
     cdef_worker[i].cdef_init_fb_row_fn = cdef_init_fb_row_fn;
+    cdef_worker[i].do_extend_border = do_extend_border;
     for (int plane = 0; plane < num_planes; plane++)
       cdef_worker[i].linebuf[plane] = cm->cdef_info.linebuf[plane];
 
@@ -1075,8 +1034,8 @@ void av1_cdef_init_fb_row_mt(const AV1_COMMON *const cm,
 void av1_cdef_frame_mt(AV1_COMMON *const cm, MACROBLOCKD *const xd,
                        AV1CdefWorkerData *const cdef_worker,
                        AVxWorker *const workers, AV1CdefSync *const cdef_sync,
-                       int num_workers,
-                       cdef_init_fb_row_t cdef_init_fb_row_fn) {
+                       int num_workers, cdef_init_fb_row_t cdef_init_fb_row_fn,
+                       int do_extend_border) {
   YV12_BUFFER_CONFIG *frame = &cm->cur_frame->buf;
   const int num_planes = av1_num_planes(cm);
 
@@ -1086,7 +1045,18 @@ void av1_cdef_frame_mt(AV1_COMMON *const cm, MACROBLOCKD *const xd,
   reset_cdef_job_info(cdef_sync);
   prepare_cdef_frame_workers(cm, xd, cdef_worker, cdef_sb_row_worker_hook,
                              workers, cdef_sync, num_workers,
-                             cdef_init_fb_row_fn);
+                             cdef_init_fb_row_fn, do_extend_border);
   launch_cdef_workers(workers, num_workers);
   sync_cdef_workers(workers, cm, num_workers);
 }
+
+int av1_get_intrabc_extra_top_right_sb_delay(const AV1_COMMON *cm) {
+  // No additional top-right delay when intraBC tool is not enabled.
+  if (!av1_allow_intrabc(cm)) return 0;
+  // Due to the hardware constraints on processing the intraBC tool with row
+  // multithreading, a top-right delay of 3 superblocks of size 128x128 or 5
+  // superblocks of size 64x64 is mandated. However, a minimum top-right delay
+  // of 1 superblock is assured with 'sync_range'. Hence return only the
+  // additional superblock delay when the intraBC tool is enabled.
+  return cm->seq_params->sb_size == BLOCK_128X128 ? 2 : 4;
+}
