@@ -23,8 +23,8 @@
 #include "av1/encoder/encoder.h"
 #include "av1/encoder/firstpass.h"
 #include "av1/encoder/gop_structure.h"
+#include "av1/encoder/pass2_strategy.h"
 
-#if CONFIG_FRAME_PARALLEL_ENCODE
 // This function sets gf_group->frame_parallel_level for LF_UPDATE frames based
 // on the value of parallel_frame_count.
 static void set_frame_parallel_level(int *frame_parallel_level,
@@ -58,7 +58,6 @@ static void set_src_offset(GF_GROUP *const gf_group, int *first_frame_index,
   }
 }
 
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
 // Sets the GF_GROUP params for LF_UPDATE frames.
 static AOM_INLINE void set_params_for_leaf_frames(
     const TWO_PASS *twopass, const TWO_PASS_FRAME *twopass_frame,
@@ -430,20 +429,15 @@ static AOM_INLINE void set_multi_layer_params_for_gf14(
     }
   }
 }
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE_2
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
 
 // Set parameters for frames between 'start' and 'end' (excluding both).
 static void set_multi_layer_params(
     const TWO_PASS *twopass, const TWO_PASS_FRAME *twopass_frame,
     GF_GROUP *const gf_group, const PRIMARY_RATE_CONTROL *p_rc,
     RATE_CONTROL *rc, FRAME_INFO *frame_info, int start, int end,
-    int *cur_frame_idx, int *frame_ind,
-#if CONFIG_FRAME_PARALLEL_ENCODE
-    int *parallel_frame_count, int max_parallel_frames,
-    int do_frame_parallel_encode, int *first_frame_index,
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
-    int layer_depth) {
+    int *cur_frame_idx, int *frame_ind, int *parallel_frame_count,
+    int max_parallel_frames, int do_frame_parallel_encode,
+    int *first_frame_index, int layer_depth) {
   const int num_frames_to_process = end - start;
 
   // Either we are at the last level of the pyramid, or we don't have enough
@@ -463,7 +457,6 @@ static void set_multi_layer_params(
       gf_group->refbuf_state[*frame_ind] = REFBUF_UPDATE;
       gf_group->max_layer_depth =
           AOMMAX(gf_group->max_layer_depth, layer_depth);
-#if CONFIG_FRAME_PARALLEL_ENCODE
       // Set the level of parallelism for the LF_UPDATE frame.
       if (do_frame_parallel_encode) {
         set_frame_parallel_level(&gf_group->frame_parallel_level[*frame_ind],
@@ -472,7 +465,6 @@ static void set_multi_layer_params(
         gf_group->is_frame_non_ref[*frame_ind] = 1;
       }
       set_src_offset(gf_group, first_frame_index, *cur_frame_idx, *frame_ind);
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
       ++(*frame_ind);
       ++(*cur_frame_idx);
       ++start;
@@ -488,7 +480,6 @@ static void set_multi_layer_params(
     gf_group->frame_type[*frame_ind] = INTER_FRAME;
     gf_group->refbuf_state[*frame_ind] = REFBUF_UPDATE;
 
-#if CONFIG_FRAME_PARALLEL_ENCODE
     if (do_frame_parallel_encode) {
       // If max_parallel_frames is not exceeded and if the frame will not be
       // temporally filtered, encode the next internal ARF frame in parallel.
@@ -500,7 +491,6 @@ static void set_multi_layer_params(
       }
     }
     set_src_offset(gf_group, first_frame_index, *cur_frame_idx, *frame_ind);
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
 
     // Get the boost factor for intermediate ARF frames.
     gf_group->arf_boost[*frame_ind] =
@@ -509,13 +499,10 @@ static void set_multi_layer_params(
     ++(*frame_ind);
 
     // Frames displayed before this internal ARF.
-    set_multi_layer_params(twopass, twopass_frame, gf_group, p_rc, rc,
-                           frame_info, start, m, cur_frame_idx, frame_ind,
-#if CONFIG_FRAME_PARALLEL_ENCODE
-                           parallel_frame_count, max_parallel_frames,
-                           do_frame_parallel_encode, first_frame_index,
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
-                           layer_depth + 1);
+    set_multi_layer_params(
+        twopass, twopass_frame, gf_group, p_rc, rc, frame_info, start, m,
+        cur_frame_idx, frame_ind, parallel_frame_count, max_parallel_frames,
+        do_frame_parallel_encode, first_frame_index, layer_depth + 1);
 
     // Overlay for internal ARF.
     gf_group->update_type[*frame_ind] = INTNL_OVERLAY_UPDATE;
@@ -526,20 +513,15 @@ static void set_multi_layer_params(
     gf_group->frame_type[*frame_ind] = INTER_FRAME;
     gf_group->refbuf_state[*frame_ind] = REFBUF_UPDATE;
 
-#if CONFIG_FRAME_PARALLEL_ENCODE
     set_src_offset(gf_group, first_frame_index, *cur_frame_idx, *frame_ind);
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
     ++(*frame_ind);
     ++(*cur_frame_idx);
 
     // Frames displayed after this internal ARF.
-    set_multi_layer_params(twopass, twopass_frame, gf_group, p_rc, rc,
-                           frame_info, m + 1, end, cur_frame_idx, frame_ind,
-#if CONFIG_FRAME_PARALLEL_ENCODE
-                           parallel_frame_count, max_parallel_frames,
-                           do_frame_parallel_encode, first_frame_index,
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
-                           layer_depth + 1);
+    set_multi_layer_params(
+        twopass, twopass_frame, gf_group, p_rc, rc, frame_info, m + 1, end,
+        cur_frame_idx, frame_ind, parallel_frame_count, max_parallel_frames,
+        do_frame_parallel_encode, first_frame_index, layer_depth + 1);
   }
 }
 
@@ -553,16 +535,11 @@ static int construct_multi_layer_gf_structure(
   int frame_index = 0;
   int cur_frame_index = 0;
 
-#if CONFIG_FRAME_PARALLEL_ENCODE
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
   // Set the display order hint for the first frame in the GF_GROUP.
   int cur_disp_index = (first_frame_update_type == KF_UPDATE)
                            ? 0
                            : cpi->common.current_frame.frame_number;
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE_2
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
 
-#if CONFIG_FRAME_PARALLEL_ENCODE
   // Initialize gf_group->frame_parallel_level and gf_group->is_frame_non_ref to
   // 0.
   memset(
@@ -572,7 +549,6 @@ static int construct_multi_layer_gf_structure(
          sizeof(gf_group->is_frame_non_ref[0]) * MAX_STATIC_GF_GROUP_LENGTH);
   memset(gf_group->src_offset, 0,
          sizeof(gf_group->src_offset[0]) * MAX_STATIC_GF_GROUP_LENGTH);
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
   // Initialize gf_group->skip_frame_refresh and gf_group->skip_frame_as_ref
   // with INVALID_IDX.
   memset(gf_group->skip_frame_refresh, INVALID_IDX,
@@ -580,8 +556,6 @@ static int construct_multi_layer_gf_structure(
              MAX_STATIC_GF_GROUP_LENGTH * REF_FRAMES);
   memset(gf_group->skip_frame_as_ref, INVALID_IDX,
          sizeof(gf_group->skip_frame_as_ref[0]) * MAX_STATIC_GF_GROUP_LENGTH);
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE_2
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
 
   int kf_decomp = cpi->oxcf.kf_cfg.enable_keyframe_filtering > 1;
   // This is a patch that fixes https://crbug.com/aomedia/3163
@@ -603,12 +577,8 @@ static int construct_multi_layer_gf_structure(
     gf_group->frame_type[frame_index] = KEY_FRAME;
     gf_group->refbuf_state[frame_index] = REFBUF_RESET;
     gf_group->max_layer_depth = 0;
-#if CONFIG_FRAME_PARALLEL_ENCODE
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
     gf_group->display_idx[frame_index] = cur_disp_index;
     if (!kf_decomp) cur_disp_index++;
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE_2
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
     ++frame_index;
 
     if (kf_decomp) {
@@ -619,12 +589,8 @@ static int construct_multi_layer_gf_structure(
       gf_group->frame_type[frame_index] = INTER_FRAME;
       gf_group->refbuf_state[frame_index] = REFBUF_UPDATE;
       gf_group->max_layer_depth = 0;
-#if CONFIG_FRAME_PARALLEL_ENCODE
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
       gf_group->display_idx[frame_index] = cur_disp_index;
       cur_disp_index++;
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE_2
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
       ++frame_index;
     }
     cur_frame_index++;
@@ -638,12 +604,8 @@ static int construct_multi_layer_gf_structure(
     gf_group->frame_type[frame_index] = INTER_FRAME;
     gf_group->refbuf_state[frame_index] = REFBUF_UPDATE;
     gf_group->max_layer_depth = 0;
-#if CONFIG_FRAME_PARALLEL_ENCODE
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
     gf_group->display_idx[frame_index] = cur_disp_index;
     cur_disp_index++;
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE_2
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
     ++frame_index;
     ++cur_frame_index;
   }
@@ -662,12 +624,8 @@ static int construct_multi_layer_gf_structure(
     gf_group->refbuf_state[frame_index] = REFBUF_UPDATE;
     gf_group->max_layer_depth = 1;
     gf_group->arf_index = frame_index;
-#if CONFIG_FRAME_PARALLEL_ENCODE
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
     gf_group->display_idx[frame_index] =
         cur_disp_index + gf_group->arf_src_offset[frame_index];
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE_2
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
     ++frame_index;
   } else {
     gf_group->arf_index = -1;
@@ -676,7 +634,6 @@ static int construct_multi_layer_gf_structure(
   // Flag to indicate if multi-layer configuration is complete.
   int is_multi_layer_configured = 0;
 
-#if CONFIG_FRAME_PARALLEL_ENCODE
   // Running count of no. of frames that is part of a given parallel
   // encode set in a gf_group. Value of 1 indicates no parallel encode.
   int parallel_frame_count = 1;
@@ -686,7 +643,6 @@ static int construct_multi_layer_gf_structure(
                                   gf_group->max_layer_depth_allowed >= 4);
 
   int first_frame_index = cur_frame_index;
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
   if (do_frame_parallel_encode) {
     // construct_multi_layer_gf_structure() takes the input parameter
     // 'gf_interval' as p_rc->baseline_gf_interval - 1 . Below code computes the
@@ -749,8 +705,9 @@ static int construct_multi_layer_gf_structure(
       }
     } else {
       // Set layer depth threshold for reordering as per the gf length.
-      int depth_thr =
-          (actual_gf_length == 16) ? 3 : (actual_gf_length == 32) ? 4 : INT_MAX;
+      int depth_thr = (actual_gf_length == 16)   ? 3
+                      : (actual_gf_length == 32) ? 4
+                                                 : INT_MAX;
 
       set_multi_layer_params_for_fp(
           twopass, &cpi->twopass_frame, gf_group, p_rc, rc, frame_info,
@@ -761,19 +718,14 @@ static int construct_multi_layer_gf_structure(
     }
     is_multi_layer_configured = 1;
   }
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE_2
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
 
   // Rest of the frames.
   if (!is_multi_layer_configured)
-    set_multi_layer_params(twopass, &cpi->twopass_frame, gf_group, p_rc, rc,
-                           frame_info, cur_frame_index, gf_interval,
-                           &cur_frame_index, &frame_index,
-#if CONFIG_FRAME_PARALLEL_ENCODE
-                           &parallel_frame_count, cpi->ppi->num_fp_contexts,
-                           do_frame_parallel_encode, &first_frame_index,
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
-                           use_altref + 1);
+    set_multi_layer_params(
+        twopass, &cpi->twopass_frame, gf_group, p_rc, rc, frame_info,
+        cur_frame_index, gf_interval, &cur_frame_index, &frame_index,
+        &parallel_frame_count, cpi->ppi->num_fp_contexts,
+        do_frame_parallel_encode, &first_frame_index, use_altref + 1);
 
   if (use_altref) {
     gf_group->update_type[frame_index] = OVERLAY_UPDATE;
@@ -784,11 +736,7 @@ static int construct_multi_layer_gf_structure(
     gf_group->frame_type[frame_index] = INTER_FRAME;
     gf_group->refbuf_state[frame_index] =
         is_fwd_kf ? REFBUF_RESET : REFBUF_UPDATE;
-#if CONFIG_FRAME_PARALLEL_ENCODE
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
     gf_group->display_idx[frame_index] = cur_disp_index;
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE_2
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
     ++frame_index;
   } else {
     for (; cur_frame_index <= gf_interval; ++cur_frame_index) {
@@ -800,18 +748,13 @@ static int construct_multi_layer_gf_structure(
       gf_group->frame_type[frame_index] = INTER_FRAME;
       gf_group->refbuf_state[frame_index] = REFBUF_UPDATE;
       gf_group->max_layer_depth = AOMMAX(gf_group->max_layer_depth, 2);
-#if CONFIG_FRAME_PARALLEL_ENCODE
       set_src_offset(gf_group, &first_frame_index, cur_frame_index,
                      frame_index);
-#if CONFIG_FRAME_PARALLEL_ENCODE_2
       gf_group->display_idx[frame_index] = cur_disp_index;
       cur_disp_index++;
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE_2
-#endif  // CONFIG_FRAME_PARALLEL_ENCODE
       ++frame_index;
     }
   }
-#if CONFIG_FRAME_PARALLEL_ENCODE
   if (do_frame_parallel_encode) {
     // Iterate through the gf_group and reset frame_parallel_level to 0 in case
     // a frame is marked as frame_parallel_level 1 with no subsequent
@@ -840,7 +783,6 @@ static int construct_multi_layer_gf_structure(
       gf_group->frame_parallel_level[frame_index - 2] = 0;
     }
   }
-#endif
 
   for (int gf_idx = frame_index; gf_idx < MAX_STATIC_GF_GROUP_LENGTH;
        ++gf_idx) {
@@ -883,10 +825,15 @@ void av1_gop_setup_structure(AV1_COMP *cpi) {
   const int key_frame = rc->frames_since_key == 0;
   FRAME_UPDATE_TYPE first_frame_update_type = ARF_UPDATE;
 
-  if (key_frame)
+  if (key_frame) {
     first_frame_update_type = KF_UPDATE;
-  else if (!cpi->ppi->gf_state.arf_gf_boost_lst)
+    if (cpi->oxcf.kf_max_pyr_height != -1) {
+      gf_group->max_layer_depth_allowed = AOMMIN(
+          cpi->oxcf.kf_max_pyr_height, gf_group->max_layer_depth_allowed);
+    }
+  } else if (!cpi->ppi->gf_state.arf_gf_boost_lst) {
     first_frame_update_type = GF_UPDATE;
+  }
 
   gf_group->size = construct_multi_layer_gf_structure(
       cpi, twopass, gf_group, rc, frame_info, p_rc->baseline_gf_interval,
