@@ -55,12 +55,16 @@ class AV1FwdTxfm2d : public ::testing::TestWithParam<AV1FwdTxfm2dParam> {
     txfm2d_size_ = tx_width_ * tx_height_;
     input_ = reinterpret_cast<int16_t *>(
         aom_memalign(16, sizeof(input_[0]) * txfm2d_size_));
+    ASSERT_NE(input_, nullptr);
     output_ = reinterpret_cast<int32_t *>(
         aom_memalign(16, sizeof(output_[0]) * txfm2d_size_));
+    ASSERT_NE(output_, nullptr);
     ref_input_ = reinterpret_cast<double *>(
         aom_memalign(16, sizeof(ref_input_[0]) * txfm2d_size_));
+    ASSERT_NE(ref_input_, nullptr);
     ref_output_ = reinterpret_cast<double *>(
         aom_memalign(16, sizeof(ref_output_[0]) * txfm2d_size_));
+    ASSERT_NE(ref_output_, nullptr);
   }
 
   void RunFwdAccuracyCheck() {
@@ -248,7 +252,7 @@ void AV1FwdTxfm2dMatchTest(TX_SIZE tx_size, lowbd_fwd_txfm_func target_func) {
     }
 
     FwdTxfm2dFunc ref_func = libaom_test::fwd_txfm_func_ls[tx_size];
-    if (ref_func != NULL) {
+    if (ref_func != nullptr) {
       DECLARE_ALIGNED(32, int16_t, input[64 * 64]) = { 0 };
       DECLARE_ALIGNED(32, int32_t, output[64 * 64]);
       DECLARE_ALIGNED(32, int32_t, ref_output[64 * 64]);
@@ -305,7 +309,7 @@ void AV1FwdTxfm2dSpeedTest(TX_SIZE tx_size, lowbd_fwd_txfm_func target_func) {
       }
 
       FwdTxfm2dFunc ref_func = libaom_test::fwd_txfm_func_ls[tx_size];
-      if (ref_func != NULL) {
+      if (ref_func != nullptr) {
         DECLARE_ALIGNED(32, int16_t, input[64 * 64]) = { 0 };
         DECLARE_ALIGNED(32, int32_t, output[64 * 64]);
         DECLARE_ALIGNED(32, int32_t, ref_output[64 * 64]);
@@ -524,7 +528,7 @@ void AV1HighbdFwdTxfm2dMatchTest(TX_SIZE tx_size,
       }
 
       FwdTxfm2dFunc ref_func = libaom_test::fwd_txfm_func_ls[tx_size];
-      if (ref_func != NULL) {
+      if (ref_func != nullptr) {
         DECLARE_ALIGNED(32, int16_t, input[64 * 64]) = { 0 };
         DECLARE_ALIGNED(32, int32_t, output[64 * 64]);
         DECLARE_ALIGNED(32, int32_t, ref_output[64 * 64]);
@@ -585,7 +589,7 @@ void AV1HighbdFwdTxfm2dSpeedTest(TX_SIZE tx_size,
       }
 
       FwdTxfm2dFunc ref_func = libaom_test::fwd_txfm_func_ls[tx_size];
-      if (ref_func != NULL) {
+      if (ref_func != nullptr) {
         DECLARE_ALIGNED(32, int16_t, input[64 * 64]) = { 0 };
         DECLARE_ALIGNED(32, int32_t, output[64 * 64]);
         DECLARE_ALIGNED(32, int32_t, ref_output[64 * 64]);
