@@ -1301,7 +1301,7 @@ const mapping m[] = { MAP(v64_sad_u8),
                       MAP(v256_unpacklo_s8_s16),
                       MAP(v256_unpackhi_s8_s16),
                       MAP(v256_blend_8),
-                      { NULL, NULL, NULL } };
+                      { nullptr, nullptr, nullptr } };
 #undef MAP
 
 // Map reference functions to machine tuned functions. Since the
@@ -1507,7 +1507,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
   memset(d, 0, sizeof(d));
 
   Map(name, &ref_simd, &simd);
-  if (simd == NULL || ref_simd == NULL) {
+  if (simd == nullptr || ref_simd == nullptr) {
     FAIL() << "Internal error: Unknown intrinsic function " << name;
   }
   for (unsigned int count = 0;
@@ -1520,7 +1520,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
 
     if (typeid(CRet) == typeid(c_v64) && typeid(CArg) == typeid(c_v64)) {
       // V64_V64
-      error = CompareSimd1Arg<v64, v64, CRet, CArg>(
+      error = CompareSimd1Arg<v64, v64, c_v64, c_v64>(
           reinterpret_cast<fptr>(v64_store_aligned),
           reinterpret_cast<fptr>(v64_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v64_store_aligned),
@@ -1528,7 +1528,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v64) &&
                typeid(CArg) == typeid(uint8_t)) {
       // V64_U8
-      error = CompareSimd1Arg<v64, uint8_t, CRet, CArg>(
+      error = CompareSimd1Arg<v64, uint8_t, c_v64, uint8_t>(
           reinterpret_cast<fptr>(v64_store_aligned),
           reinterpret_cast<fptr>(u8_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v64_store_aligned),
@@ -1536,7 +1536,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v64) &&
                typeid(CArg) == typeid(uint16_t)) {
       // V64_U16
-      error = CompareSimd1Arg<v64, uint16_t, CRet, CArg>(
+      error = CompareSimd1Arg<v64, uint16_t, c_v64, uint16_t>(
           reinterpret_cast<fptr>(v64_store_aligned),
           reinterpret_cast<fptr>(u16_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v64_store_aligned),
@@ -1544,7 +1544,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v64) &&
                typeid(CArg) == typeid(uint32_t)) {
       // V64_U32
-      error = CompareSimd1Arg<v64, uint32_t, CRet, CArg>(
+      error = CompareSimd1Arg<v64, uint32_t, c_v64, uint32_t>(
           reinterpret_cast<fptr>(v64_store_aligned),
           reinterpret_cast<fptr>(u32_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v64_store_aligned),
@@ -1552,7 +1552,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(uint64_t) &&
                typeid(CArg) == typeid(c_v64)) {
       // U64_V64
-      error = CompareSimd1Arg<uint64_t, v64, CRet, CArg>(
+      error = CompareSimd1Arg<uint64_t, v64, uint64_t, c_v64>(
           reinterpret_cast<fptr>(u64_store_aligned),
           reinterpret_cast<fptr>(v64_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_u64_store_aligned),
@@ -1560,7 +1560,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(int64_t) &&
                typeid(CArg) == typeid(c_v64)) {
       // S64_V64
-      error = CompareSimd1Arg<int64_t, v64, CRet, CArg>(
+      error = CompareSimd1Arg<int64_t, v64, int64_t, c_v64>(
           reinterpret_cast<fptr>(s64_store_aligned),
           reinterpret_cast<fptr>(v64_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_s64_store_aligned),
@@ -1568,7 +1568,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(uint32_t) &&
                typeid(CArg) == typeid(c_v64)) {
       // U32_V64
-      error = CompareSimd1Arg<uint32_t, v64, CRet, CArg>(
+      error = CompareSimd1Arg<uint32_t, v64, uint32_t, c_v64>(
           reinterpret_cast<fptr>(u32_store_aligned),
           reinterpret_cast<fptr>(v64_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_u32_store_aligned),
@@ -1576,7 +1576,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(int32_t) &&
                typeid(CArg) == typeid(c_v64)) {
       // S32_V64
-      error = CompareSimd1Arg<int32_t, v64, CRet, CArg>(
+      error = CompareSimd1Arg<int32_t, v64, int32_t, c_v64>(
           reinterpret_cast<fptr>(s32_store_aligned),
           reinterpret_cast<fptr>(v64_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_s32_store_aligned),
@@ -1584,7 +1584,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(uint32_t) &&
                typeid(CArg) == typeid(c_v128)) {
       // U32_V128
-      error = CompareSimd1Arg<uint32_t, v128, CRet, CArg>(
+      error = CompareSimd1Arg<uint32_t, v128, uint32_t, c_v128>(
           reinterpret_cast<fptr>(u32_store_aligned),
           reinterpret_cast<fptr>(v128_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_u32_store_aligned),
@@ -1592,7 +1592,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(uint64_t) &&
                typeid(CArg) == typeid(c_v128)) {
       // U64_V128
-      error = CompareSimd1Arg<uint64_t, v128, CRet, CArg>(
+      error = CompareSimd1Arg<uint64_t, v128, uint64_t, c_v128>(
           reinterpret_cast<fptr>(u64_store_aligned),
           reinterpret_cast<fptr>(v128_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_u64_store_aligned),
@@ -1600,7 +1600,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(uint64_t) &&
                typeid(CArg) == typeid(c_v256)) {
       // U64_V256
-      error = CompareSimd1Arg<uint64_t, v256, CRet, CArg>(
+      error = CompareSimd1Arg<uint64_t, v256, uint64_t, c_v256>(
           reinterpret_cast<fptr>(u64_store_aligned),
           reinterpret_cast<fptr>(v256_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_u64_store_aligned),
@@ -1608,7 +1608,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v64) &&
                typeid(CArg) == typeid(c_v128)) {
       // V64_V128
-      error = CompareSimd1Arg<v64, v128, CRet, CArg>(
+      error = CompareSimd1Arg<v64, v128, c_v64, c_v128>(
           reinterpret_cast<fptr>(v64_store_aligned),
           reinterpret_cast<fptr>(v128_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v64_store_aligned),
@@ -1616,7 +1616,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v128) &&
                typeid(CArg) == typeid(c_v128)) {
       // V128_V128
-      error = CompareSimd1Arg<v128, v128, CRet, CArg>(
+      error = CompareSimd1Arg<v128, v128, c_v128, c_v128>(
           reinterpret_cast<fptr>(v128_store_aligned),
           reinterpret_cast<fptr>(v128_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v128_store_aligned),
@@ -1624,7 +1624,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v128) &&
                typeid(CArg) == typeid(c_v64)) {
       // V128_V64
-      error = CompareSimd1Arg<v128, v64, CRet, CArg>(
+      error = CompareSimd1Arg<v128, v64, c_v128, c_v64>(
           reinterpret_cast<fptr>(v128_store_aligned),
           reinterpret_cast<fptr>(v64_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v128_store_aligned),
@@ -1632,7 +1632,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v128) &&
                typeid(CArg) == typeid(uint8_t)) {
       // V128_U8
-      error = CompareSimd1Arg<v128, uint8_t, CRet, CArg>(
+      error = CompareSimd1Arg<v128, uint8_t, c_v128, uint8_t>(
           reinterpret_cast<fptr>(v128_store_aligned),
           reinterpret_cast<fptr>(u8_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v128_store_aligned),
@@ -1640,7 +1640,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v128) &&
                typeid(CArg) == typeid(uint16_t)) {
       // V128_U16
-      error = CompareSimd1Arg<v128, uint16_t, CRet, CArg>(
+      error = CompareSimd1Arg<v128, uint16_t, c_v128, uint16_t>(
           reinterpret_cast<fptr>(v128_store_aligned),
           reinterpret_cast<fptr>(u16_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v128_store_aligned),
@@ -1648,7 +1648,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v128) &&
                typeid(CArg) == typeid(uint32_t)) {
       // V128_U32
-      error = CompareSimd1Arg<v128, uint32_t, CRet, CArg>(
+      error = CompareSimd1Arg<v128, uint32_t, c_v128, uint32_t>(
           reinterpret_cast<fptr>(v128_store_aligned),
           reinterpret_cast<fptr>(u32_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v128_store_aligned),
@@ -1656,7 +1656,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v128) &&
                typeid(CArg) == typeid(uint64_t)) {
       // V128_U64
-      error = CompareSimd1Arg<v128, uint64_t, CRet, CArg>(
+      error = CompareSimd1Arg<v128, uint64_t, c_v128, uint64_t>(
           reinterpret_cast<fptr>(v128_store_aligned),
           reinterpret_cast<fptr>(u64_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v128_store_aligned),
@@ -1664,7 +1664,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v256) &&
                typeid(CArg) == typeid(c_v256)) {
       // V256_V256
-      error = CompareSimd1Arg<v256, v256, CRet, CArg>(
+      error = CompareSimd1Arg<v256, v256, c_v256, c_v256>(
           reinterpret_cast<fptr>(v256_store_aligned),
           reinterpret_cast<fptr>(v256_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v256_store_aligned),
@@ -1672,7 +1672,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v256) &&
                typeid(CArg) == typeid(c_v128)) {
       // V256_V128
-      error = CompareSimd1Arg<v256, v128, CRet, CArg>(
+      error = CompareSimd1Arg<v256, v128, c_v256, c_v128>(
           reinterpret_cast<fptr>(v256_store_aligned),
           reinterpret_cast<fptr>(v128_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v256_store_aligned),
@@ -1680,7 +1680,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v256) &&
                typeid(CArg) == typeid(uint8_t)) {
       // V256_U8
-      error = CompareSimd1Arg<v256, uint8_t, CRet, CArg>(
+      error = CompareSimd1Arg<v256, uint8_t, c_v256, uint8_t>(
           reinterpret_cast<fptr>(v256_store_aligned),
           reinterpret_cast<fptr>(u8_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v256_store_aligned),
@@ -1688,7 +1688,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v256) &&
                typeid(CArg) == typeid(uint16_t)) {
       // V256_U16
-      error = CompareSimd1Arg<v256, uint16_t, CRet, CArg>(
+      error = CompareSimd1Arg<v256, uint16_t, c_v256, uint16_t>(
           reinterpret_cast<fptr>(v256_store_aligned),
           reinterpret_cast<fptr>(u16_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v256_store_aligned),
@@ -1696,7 +1696,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v256) &&
                typeid(CArg) == typeid(uint32_t)) {
       // V256_U32
-      error = CompareSimd1Arg<v256, uint32_t, CRet, CArg>(
+      error = CompareSimd1Arg<v256, uint32_t, c_v256, uint32_t>(
           reinterpret_cast<fptr>(v256_store_aligned),
           reinterpret_cast<fptr>(u32_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v256_store_aligned),
@@ -1704,7 +1704,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v256) &&
                typeid(CArg) == typeid(uint64_t)) {
       // V256_U64
-      error = CompareSimd1Arg<v256, uint64_t, CRet, CArg>(
+      error = CompareSimd1Arg<v256, uint64_t, c_v256, uint64_t>(
           reinterpret_cast<fptr>(v256_store_aligned),
           reinterpret_cast<fptr>(u64_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v256_store_aligned),
@@ -1712,7 +1712,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(uint32_t) &&
                typeid(CArg) == typeid(c_v256)) {
       // U32_V256
-      error = CompareSimd1Arg<uint32_t, v256, CRet, CArg>(
+      error = CompareSimd1Arg<uint32_t, v256, uint32_t, c_v256>(
           reinterpret_cast<fptr>(u32_store_aligned),
           reinterpret_cast<fptr>(v256_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_u32_store_aligned),
@@ -1720,7 +1720,7 @@ void TestSimd1Arg(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     } else if (typeid(CRet) == typeid(c_v64) &&
                typeid(CArg) == typeid(c_v256)) {
       // V64_V256
-      error = CompareSimd1Arg<v64, v256, CRet, CArg>(
+      error = CompareSimd1Arg<v64, v256, c_v64, c_v256>(
           reinterpret_cast<fptr>(v64_store_aligned),
           reinterpret_cast<fptr>(v256_load_aligned), simd, d,
           reinterpret_cast<fptr>(c_v64_store_aligned),
@@ -1754,7 +1754,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
   memset(d, 0, sizeof(d));
 
   Map(name, &ref_simd, &simd);
-  if (simd == NULL || ref_simd == NULL) {
+  if (simd == nullptr || ref_simd == nullptr) {
     FAIL() << "Internal error: Unknown intrinsic function " << name;
   }
 
@@ -1769,7 +1769,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     if (typeid(CRet) == typeid(c_v64) && typeid(CArg1) == typeid(c_v64) &&
         typeid(CArg2) == typeid(c_v64)) {
       // V64_V64V64
-      error = CompareSimd2Args<v64, v64, v64, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<v64, v64, v64, c_v64, c_v64, c_v64>(
           reinterpret_cast<fptr>(v64_store_aligned),
           reinterpret_cast<fptr>(v64_load_aligned),
           reinterpret_cast<fptr>(v64_load_aligned), simd, d,
@@ -1781,19 +1781,20 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(uint32_t) &&
                typeid(CArg2) == typeid(uint32_t)) {
       // V64_U32U32
-      error = CompareSimd2Args<v64, uint32_t, uint32_t, CRet, CArg1, CArg2>(
-          reinterpret_cast<fptr>(v64_store_aligned),
-          reinterpret_cast<fptr>(u32_load_aligned),
-          reinterpret_cast<fptr>(u32_load_aligned), simd, d,
-          reinterpret_cast<fptr>(c_v64_store_aligned),
-          reinterpret_cast<fptr>(c_u32_load_aligned),
-          reinterpret_cast<fptr>(c_u32_load_aligned),
-          reinterpret_cast<fptr>(ref_simd), ref_d, s1, s2);
+      error =
+          CompareSimd2Args<v64, uint32_t, uint32_t, c_v64, uint32_t, uint32_t>(
+              reinterpret_cast<fptr>(v64_store_aligned),
+              reinterpret_cast<fptr>(u32_load_aligned),
+              reinterpret_cast<fptr>(u32_load_aligned), simd, d,
+              reinterpret_cast<fptr>(c_v64_store_aligned),
+              reinterpret_cast<fptr>(c_u32_load_aligned),
+              reinterpret_cast<fptr>(c_u32_load_aligned),
+              reinterpret_cast<fptr>(ref_simd), ref_d, s1, s2);
     } else if (typeid(CRet) == typeid(uint32_t) &&
                typeid(CArg1) == typeid(c_v64) &&
                typeid(CArg2) == typeid(c_v64)) {
       // U32_V64V64
-      error = CompareSimd2Args<uint32_t, v64, v64, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<uint32_t, v64, v64, uint32_t, c_v64, c_v64>(
           reinterpret_cast<fptr>(u32_store_aligned),
           reinterpret_cast<fptr>(v64_load_aligned),
           reinterpret_cast<fptr>(v64_load_aligned), simd, d,
@@ -1805,7 +1806,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v64) &&
                typeid(CArg2) == typeid(c_v64)) {
       // S64_V64V64
-      error = CompareSimd2Args<int64_t, v64, v64, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<int64_t, v64, v64, int64_t, c_v64, c_v64>(
           reinterpret_cast<fptr>(s64_store_aligned),
           reinterpret_cast<fptr>(v64_load_aligned),
           reinterpret_cast<fptr>(v64_load_aligned), simd, d,
@@ -1817,7 +1818,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v64) &&
                typeid(CArg2) == typeid(uint32_t)) {
       // V64_V64U32
-      error = CompareSimd2Args<v64, v64, uint32_t, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<v64, v64, uint32_t, c_v64, c_v64, uint32_t>(
           reinterpret_cast<fptr>(v64_store_aligned),
           reinterpret_cast<fptr>(v64_load_aligned),
           reinterpret_cast<fptr>(u32_load_aligned), simd, d,
@@ -1829,7 +1830,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v128) &&
                typeid(CArg2) == typeid(c_v128)) {
       // V128_V128V128
-      error = CompareSimd2Args<v128, v128, v128, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<v128, v128, v128, c_v128, c_v128, c_v128>(
           reinterpret_cast<fptr>(v128_store_aligned),
           reinterpret_cast<fptr>(v128_load_aligned),
           reinterpret_cast<fptr>(v128_load_aligned), simd, d,
@@ -1841,7 +1842,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v128) &&
                typeid(CArg2) == typeid(c_v128)) {
       // U32_V128V128
-      error = CompareSimd2Args<uint32_t, v128, v128, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<uint32_t, v128, v128, uint32_t, c_v128, c_v128>(
           reinterpret_cast<fptr>(u32_store_aligned),
           reinterpret_cast<fptr>(v128_load_aligned),
           reinterpret_cast<fptr>(v128_load_aligned), simd, d,
@@ -1853,7 +1854,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v128) &&
                typeid(CArg2) == typeid(c_v128)) {
       // U64_V128V128
-      error = CompareSimd2Args<uint64_t, v128, v128, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<uint64_t, v128, v128, uint64_t, c_v128, c_v128>(
           reinterpret_cast<fptr>(u64_store_aligned),
           reinterpret_cast<fptr>(v128_load_aligned),
           reinterpret_cast<fptr>(v128_load_aligned), simd, d,
@@ -1865,7 +1866,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v128) &&
                typeid(CArg2) == typeid(c_v128)) {
       // S64_V128V128
-      error = CompareSimd2Args<int64_t, v128, v128, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<int64_t, v128, v128, int64_t, c_v128, c_v128>(
           reinterpret_cast<fptr>(s64_store_aligned),
           reinterpret_cast<fptr>(v128_load_aligned),
           reinterpret_cast<fptr>(v128_load_aligned), simd, d,
@@ -1877,7 +1878,8 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(uint64_t) &&
                typeid(CArg2) == typeid(uint64_t)) {
       // V128_U64U64
-      error = CompareSimd2Args<v128, uint64_t, uint64_t, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<v128, uint64_t, uint64_t, c_v128, uint64_t,
+                               uint64_t>(
           reinterpret_cast<fptr>(v128_store_aligned),
           reinterpret_cast<fptr>(u64_load_aligned),
           reinterpret_cast<fptr>(u64_load_aligned), simd, d,
@@ -1889,7 +1891,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v64) &&
                typeid(CArg2) == typeid(c_v64)) {
       // V128_V64V64
-      error = CompareSimd2Args<v128, v64, v64, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<v128, v64, v64, c_v128, c_v64, c_v64>(
           reinterpret_cast<fptr>(v128_store_aligned),
           reinterpret_cast<fptr>(v64_load_aligned),
           reinterpret_cast<fptr>(v64_load_aligned), simd, d,
@@ -1901,7 +1903,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v128) &&
                typeid(CArg2) == typeid(uint32_t)) {
       // V128_V128U32
-      error = CompareSimd2Args<v128, v128, uint32_t, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<v128, v128, uint32_t, c_v128, c_v128, uint32_t>(
           reinterpret_cast<fptr>(v128_store_aligned),
           reinterpret_cast<fptr>(v128_load_aligned),
           reinterpret_cast<fptr>(u32_load_aligned), simd, d,
@@ -1913,7 +1915,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v256) &&
                typeid(CArg2) == typeid(c_v256)) {
       // V256_V256V256
-      error = CompareSimd2Args<v256, v256, v256, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<v256, v256, v256, c_v256, c_v256, c_v256>(
           reinterpret_cast<fptr>(v256_store_aligned),
           reinterpret_cast<fptr>(v256_load_aligned),
           reinterpret_cast<fptr>(v256_load_aligned), simd, d,
@@ -1925,7 +1927,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v256) &&
                typeid(CArg2) == typeid(c_v256)) {
       // U64_V256V256
-      error = CompareSimd2Args<uint64_t, v256, v256, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<uint64_t, v256, v256, uint64_t, c_v256, c_v256>(
           reinterpret_cast<fptr>(u64_store_aligned),
           reinterpret_cast<fptr>(v256_load_aligned),
           reinterpret_cast<fptr>(v256_load_aligned), simd, d,
@@ -1937,7 +1939,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v256) &&
                typeid(CArg2) == typeid(c_v256)) {
       // S64_V256V256
-      error = CompareSimd2Args<int64_t, v256, v256, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<int64_t, v256, v256, int64_t, c_v256, c_v256>(
           reinterpret_cast<fptr>(s64_store_aligned),
           reinterpret_cast<fptr>(v256_load_aligned),
           reinterpret_cast<fptr>(v256_load_aligned), simd, d,
@@ -1949,7 +1951,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v256) &&
                typeid(CArg2) == typeid(c_v256)) {
       // U32_V256V256
-      error = CompareSimd2Args<uint32_t, v256, v256, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<uint32_t, v256, v256, uint32_t, c_v256, c_v256>(
           reinterpret_cast<fptr>(u32_store_aligned),
           reinterpret_cast<fptr>(v256_load_aligned),
           reinterpret_cast<fptr>(v256_load_aligned), simd, d,
@@ -1961,7 +1963,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v128) &&
                typeid(CArg2) == typeid(c_v128)) {
       // V256_V128V128
-      error = CompareSimd2Args<v256, v128, v128, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<v256, v128, v128, c_v256, c_v128, c_v128>(
           reinterpret_cast<fptr>(v256_store_aligned),
           reinterpret_cast<fptr>(v128_load_aligned),
           reinterpret_cast<fptr>(v128_load_aligned), simd, d,
@@ -1973,7 +1975,7 @@ void TestSimd2Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
                typeid(CArg1) == typeid(c_v256) &&
                typeid(CArg2) == typeid(uint32_t)) {
       // V256_V256U32
-      error = CompareSimd2Args<v256, v256, uint32_t, CRet, CArg1, CArg2>(
+      error = CompareSimd2Args<v256, v256, uint32_t, c_v256, c_v256, uint32_t>(
           reinterpret_cast<fptr>(v256_store_aligned),
           reinterpret_cast<fptr>(v256_load_aligned),
           reinterpret_cast<fptr>(u32_load_aligned), simd, d,
@@ -2014,7 +2016,7 @@ void TestSimd3Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
   memset(d, 0, sizeof(d));
 
   Map(name, &ref_simd, &simd);
-  if (simd == NULL || ref_simd == NULL) {
+  if (simd == nullptr || ref_simd == nullptr) {
     FAIL() << "Internal error: Unknown intrinsic function " << name;
   }
 
@@ -2031,33 +2033,33 @@ void TestSimd3Args(uint32_t iterations, uint32_t mask, uint32_t maskwidth,
     if (typeid(CRet) == typeid(c_v128) && typeid(CArg1) == typeid(c_v128) &&
         typeid(CArg2) == typeid(c_v128) && typeid(CArg3) == typeid(c_v128)) {
       // V128_V128V128V128
-      error =
-          CompareSimd3Args<v128, v128, v128, v128, CRet, CArg1, CArg2, CArg3>(
-              reinterpret_cast<fptr>(v128_store_aligned),
-              reinterpret_cast<fptr>(v128_load_aligned),
-              reinterpret_cast<fptr>(v128_load_aligned),
-              reinterpret_cast<fptr>(v128_load_aligned), simd, d,
-              reinterpret_cast<fptr>(c_v128_store_aligned),
-              reinterpret_cast<fptr>(c_v128_load_aligned),
-              reinterpret_cast<fptr>(c_v128_load_aligned),
-              reinterpret_cast<fptr>(c_v128_load_aligned),
-              reinterpret_cast<fptr>(ref_simd), ref_d, s1, s2, s3);
+      error = CompareSimd3Args<v128, v128, v128, v128, c_v128, c_v128, c_v128,
+                               c_v128>(
+          reinterpret_cast<fptr>(v128_store_aligned),
+          reinterpret_cast<fptr>(v128_load_aligned),
+          reinterpret_cast<fptr>(v128_load_aligned),
+          reinterpret_cast<fptr>(v128_load_aligned), simd, d,
+          reinterpret_cast<fptr>(c_v128_store_aligned),
+          reinterpret_cast<fptr>(c_v128_load_aligned),
+          reinterpret_cast<fptr>(c_v128_load_aligned),
+          reinterpret_cast<fptr>(c_v128_load_aligned),
+          reinterpret_cast<fptr>(ref_simd), ref_d, s1, s2, s3);
     } else if (typeid(CRet) == typeid(c_v256) &&
                typeid(CArg1) == typeid(c_v256) &&
                typeid(CArg2) == typeid(c_v256) &&
                typeid(CArg3) == typeid(c_v256)) {
       // V256_V256V256V256
-      error =
-          CompareSimd3Args<v256, v256, v256, v256, CRet, CArg1, CArg2, CArg3>(
-              reinterpret_cast<fptr>(v256_store_aligned),
-              reinterpret_cast<fptr>(v256_load_aligned),
-              reinterpret_cast<fptr>(v256_load_aligned),
-              reinterpret_cast<fptr>(v256_load_aligned), simd, d,
-              reinterpret_cast<fptr>(c_v256_store_aligned),
-              reinterpret_cast<fptr>(c_v256_load_aligned),
-              reinterpret_cast<fptr>(c_v256_load_aligned),
-              reinterpret_cast<fptr>(c_v256_load_aligned),
-              reinterpret_cast<fptr>(ref_simd), ref_d, s1, s2, s3);
+      error = CompareSimd3Args<v256, v256, v256, v256, c_v256, c_v256, c_v256,
+                               c_v256>(
+          reinterpret_cast<fptr>(v256_store_aligned),
+          reinterpret_cast<fptr>(v256_load_aligned),
+          reinterpret_cast<fptr>(v256_load_aligned),
+          reinterpret_cast<fptr>(v256_load_aligned), simd, d,
+          reinterpret_cast<fptr>(c_v256_store_aligned),
+          reinterpret_cast<fptr>(c_v256_load_aligned),
+          reinterpret_cast<fptr>(c_v256_load_aligned),
+          reinterpret_cast<fptr>(c_v256_load_aligned),
+          reinterpret_cast<fptr>(ref_simd), ref_d, s1, s2, s3);
     } else {
       FAIL() << "Internal error: Unknown intrinsic function "
              << typeid(CRet).name() << " " << name << "("
