@@ -10,6 +10,8 @@
  */
 
 #include <cstdlib>
+#include <memory>
+#include <new>
 #include <vector>
 
 #include "av1/encoder/cost.h"
@@ -19,6 +21,10 @@
 
 namespace {
 
+#if CONFIG_BITRATE_ACCURACY
+constexpr double epsilon = 0.0000001;
+#endif
+
 double laplace_prob(double q_step, double b, double zero_bin_ratio,
                     int qcoeff) {
   int abs_qcoeff = abs(qcoeff);
@@ -70,6 +76,26 @@ TEST(TplModelTest, TransformCoeffEntropyTest2) {
   EXPECT_NEAR(expected_rate, est_expected_rate, 0.001);
 }
 
+TEST(TplModelTest, InitTplStats1) {
+  // We use heap allocation instead of stack allocation here to avoid
+  // -Wstack-usage warning.
+  std::unique_ptr<TplParams> tpl_data(new (std::nothrow) TplParams);
+  ASSERT_NE(tpl_data, nullptr);
+  av1_zero(*tpl_data);
+  tpl_data->ready = 1;
+  EXPECT_EQ(sizeof(tpl_data->tpl_stats_buffer),
+            MAX_LENGTH_TPL_FRAME_STATS * sizeof(tpl_data->tpl_stats_buffer[0]));
+  for (int i = 0; i < MAX_LENGTH_TPL_FRAME_STATS; ++i) {
+    // Set it to a random non-zero number
+    tpl_data->tpl_stats_buffer[i].is_valid = i + 1;
+  }
+  av1_init_tpl_stats(tpl_data.get());
+  EXPECT_EQ(tpl_data->ready, 0);
+  for (int i = 0; i < MAX_LENGTH_TPL_FRAME_STATS; ++i) {
+    EXPECT_EQ(tpl_data->tpl_stats_buffer[i].is_valid, 0);
+  }
+}
+
 TEST(TplModelTest, DeltaRateCostZeroFlow) {
   // When srcrf_dist equal to recrf_dist, av1_delta_rate_cost should return 0
   int64_t srcrf_dist = 256;
@@ -147,34 +173,26 @@ TEST(TplModelTest, GetOverlapAreaNoOverlap) {
   EXPECT_EQ(overlap_area, 0);
 }
 
-TEST(TPLModelTest, EstimateFrameRateTest) {
-  /*
-   * Transform size: 16x16
-   * Frame count: 16
-   * Transform block count: 20
-   */
-  const int txfm_size = 256;  // 16x16
-  const int frame_count = 16;
-  int q_index_list[16];
-  int valid_list[16];
-  TplTxfmStats stats_list[16];
-
-  for (int i = 0; i < frame_count; i++) {
-    q_index_list[i] = 1;
-    valid_list[i] = 1;
-    stats_list[i].txfm_block_count = 8;
-
-    for (int j = 0; j < txfm_size; j++) {
-      stats_list[i].abs_coeff_sum[j] = 0;
-    }
+TEST(TplModelTest, GetQIndexFromQstepRatio) {
+  const aom_bit_depth_t bit_depth = AOM_BITS_8;
+  // When qstep_ratio is 1, the output q_index should be equal to leaf_qindex.
+  double qstep_ratio = 1.0;
+  for (int leaf_qindex = 1; leaf_qindex <= 255; ++leaf_qindex) {
+    const int q_index =
+        av1_get_q_index_from_qstep_ratio(leaf_qindex, qstep_ratio, bit_depth);
+    EXPECT_EQ(q_index, leaf_qindex);
   }
 
-  double result = av1_estimate_gop_bitrate(q_index_list, frame_count,
-                                           stats_list, valid_list, NULL);
-  EXPECT_NEAR(result, 0, 0.1);
+  // When qstep_ratio is very low, the output q_index should be 1.
+  qstep_ratio = 0.0001;
+  for (int leaf_qindex = 1; leaf_qindex <= 255; ++leaf_qindex) {
+    const int q_index =
+        av1_get_q_index_from_qstep_ratio(leaf_qindex, qstep_ratio, bit_depth);
+    EXPECT_EQ(q_index, 0);
+  }
 }
 
-TEST(TPLModelTest, TxfmStatsInitTest) {
+TEST(TplModelTest, TxfmStatsInitTest) {
   TplTxfmStats tpl_txfm_stats;
   av1_init_tpl_txfm_stats(&tpl_txfm_stats);
   EXPECT_EQ(tpl_txfm_stats.coeff_num, 256);
@@ -184,7 +202,7 @@ TEST(TPLModelTest, TxfmStatsInitTest) {
   }
 }
 
-TEST(TPLModelTest, TxfmStatsAccumulateTest) {
+TEST(TplModelTest, TxfmStatsAccumulateTest) {
   TplTxfmStats sub_stats;
   av1_init_tpl_txfm_stats(&sub_stats);
   sub_stats.txfm_block_count = 17;
@@ -206,7 +224,7 @@ TEST(TPLModelTest, TxfmStatsAccumulateTest) {
   }
 }
 
-TEST(TPLModelTest, TxfmStatsRecordTest) {
+TEST(TplModelTest, TxfmStatsRecordTest) {
   TplTxfmStats stats1;
   TplTxfmStats stats2;
   av1_init_tpl_txfm_stats(&stats1);
@@ -231,91 +249,6 @@ TEST(TPLModelTest, TxfmStatsRecordTest) {
   }
 }
 
-/*
- * Helper method to brute-force search for the closest q_index
- * that achieves the specified bit budget.
- */
-int find_gop_q_iterative(double bit_budget, const double *qstep_ratio_list,
-                         GF_GROUP gf_group, const int *stats_valid_list,
-                         TplTxfmStats *stats_list, int gf_frame_index,
-                         aom_bit_depth_t bit_depth) {
-  // Brute force iterative method to find the optimal q.
-  // Use the result to test against the binary search result.
-
-  // Initial estimate when q = 255
-  av1_q_mode_compute_gop_q_indices(gf_frame_index, 255, qstep_ratio_list,
-                                   bit_depth, &gf_group, gf_group.q_val);
-  double curr_estimate = av1_estimate_gop_bitrate(
-      gf_group.q_val, gf_group.size, stats_list, stats_valid_list, NULL);
-  double best_estimate_budget_distance = fabs(curr_estimate - bit_budget);
-  int best_q = 255;
-
-  // Start at q = 254 because we already have an estimate for q = 255.
-  for (int q = 254; q >= 0; q--) {
-    av1_q_mode_compute_gop_q_indices(gf_frame_index, q, qstep_ratio_list,
-                                     bit_depth, &gf_group, gf_group.q_val);
-    curr_estimate = av1_estimate_gop_bitrate(
-        gf_group.q_val, gf_group.size, stats_list, stats_valid_list, NULL);
-    double curr_estimate_budget_distance = fabs(curr_estimate - bit_budget);
-    if (curr_estimate_budget_distance <= best_estimate_budget_distance) {
-      best_estimate_budget_distance = curr_estimate_budget_distance;
-      best_q = q;
-    }
-  }
-  return best_q;
-}
-
-TEST(TplModelTest, QModeEstimateBaseQTest) {
-  GF_GROUP gf_group = {};
-  gf_group.size = 25;
-  TplTxfmStats stats_list[25];
-  int q_index_list[25];
-  const int gf_group_update_types[25] = { 0, 3, 6, 6, 6, 1, 5, 1, 5, 6, 1, 5, 1,
-                                          5, 6, 6, 1, 5, 1, 5, 6, 1, 5, 1, 4 };
-  int stats_valid_list[25] = { 0 };
-  const int gf_frame_index = 0;
-  const aom_bit_depth_t bit_depth = AOM_BITS_8;
-  const double scale_factor = 1.0;
-
-  double qstep_ratio_list[25];
-  for (int i = 0; i < 25; i++) {
-    qstep_ratio_list[i] = 1;
-  }
-
-  for (int i = 0; i < gf_group.size; i++) {
-    stats_valid_list[i] = 1;
-    gf_group.update_type[i] = gf_group_update_types[i];
-    stats_list[i].txfm_block_count = 8;
-
-    for (int j = 0; j < 256; j++) {
-      stats_list[i].abs_coeff_sum[j] = 1000 + j;
-    }
-  }
-
-  // Test multiple bit budgets.
-  const std::vector<double> bit_budgets = { 0,      100,    1000,   10000,
-                                            100000, 300000, 500000, 750000,
-                                            800000, DBL_MAX };
-
-  for (double bit_budget : bit_budgets) {
-    // Binary search method to find the optimal q.
-    const int result = av1_q_mode_estimate_base_q(
-        &gf_group, stats_list, stats_valid_list, bit_budget, gf_frame_index,
-        bit_depth, scale_factor, qstep_ratio_list, q_index_list, NULL);
-    const int test_result = find_gop_q_iterative(
-        bit_budget, qstep_ratio_list, gf_group, stats_valid_list, stats_list,
-        gf_frame_index, bit_depth);
-
-    if (bit_budget == 0) {
-      EXPECT_EQ(result, 255);
-    } else if (bit_budget == DBL_MAX) {
-      EXPECT_EQ(result, 0);
-    }
-
-    EXPECT_EQ(result, test_result);
-  }
-}
-
 TEST(TplModelTest, ComputeMVDifferenceTest) {
   TplDepFrame tpl_frame_small;
   tpl_frame_small.is_valid = true;
@@ -422,5 +355,173 @@ TEST(TplModelTest, ComputeMVBitsTest) {
   // Neighboring vectors will have different values, increasing the cost.
   EXPECT_NEAR(result, 70, 5);
 }
+#if CONFIG_BITRATE_ACCURACY
+
+TEST(TplModelTest, VbrRcInfoSetGopBitBudget) {
+  VBR_RATECTRL_INFO vbr_rc_info;
+  const double total_bit_budget = 2000;
+  const int show_frame_count = 8;
+  const int gop_show_frame_count = 4;
+  av1_vbr_rc_init(&vbr_rc_info, total_bit_budget, show_frame_count);
+  av1_vbr_rc_set_gop_bit_budget(&vbr_rc_info, gop_show_frame_count);
+  EXPECT_NEAR(vbr_rc_info.gop_bit_budget, 1000, epsilon);
+}
+
+void init_toy_gf_group(GF_GROUP *gf_group) {
+  av1_zero(*gf_group);
+  gf_group->size = 4;
+  const FRAME_UPDATE_TYPE update_type[4] = { KF_UPDATE, ARF_UPDATE,
+                                             INTNL_ARF_UPDATE, LF_UPDATE };
+  for (int i = 0; i < gf_group->size; ++i) {
+    gf_group->update_type[i] = update_type[i];
+  }
+}
+
+void init_toy_vbr_rc_info(VBR_RATECTRL_INFO *vbr_rc_info, int gop_size) {
+  int total_bit_budget = 2000;
+  int show_frame_count = 8;
+  av1_vbr_rc_init(vbr_rc_info, total_bit_budget, show_frame_count);
+
+  for (int i = 0; i < gop_size; ++i) {
+    vbr_rc_info->qstep_ratio_list[i] = 1;
+  }
+}
+
+void init_toy_tpl_txfm_stats(std::vector<TplTxfmStats> *stats_list) {
+  for (size_t i = 0; i < stats_list->size(); i++) {
+    TplTxfmStats *txfm_stats = &stats_list->at(i);
+    av1_init_tpl_txfm_stats(txfm_stats);
+    txfm_stats->txfm_block_count = 8;
+    for (int j = 0; j < txfm_stats->coeff_num; j++) {
+      txfm_stats->abs_coeff_sum[j] = 1000 + j;
+    }
+    av1_tpl_txfm_stats_update_abs_coeff_mean(txfm_stats);
+  }
+}
+
+/*
+ * Helper method to brute-force search for the closest q_index
+ * that achieves the specified bit budget.
+ */
+int find_gop_q_iterative(double bit_budget, aom_bit_depth_t bit_depth,
+                         const double *update_type_scale_factors,
+                         int frame_count,
+                         const FRAME_UPDATE_TYPE *update_type_list,
+                         const double *qstep_ratio_list,
+                         const TplTxfmStats *stats_list, int *q_index_list,
+                         double *estimated_bitrate_byframe) {
+  int best_q = 255;
+  double curr_estimate = av1_vbr_rc_info_estimate_gop_bitrate(
+      best_q, bit_depth, update_type_scale_factors, frame_count,
+      update_type_list, qstep_ratio_list, stats_list, q_index_list,
+      estimated_bitrate_byframe);
+  double min_bits_diff = fabs(curr_estimate - bit_budget);
+  // Start at q = 254 because we already have an estimate for q = 255.
+  for (int q = 254; q >= 0; q--) {
+    double curr_estimate = av1_vbr_rc_info_estimate_gop_bitrate(
+        q, bit_depth, update_type_scale_factors, frame_count, update_type_list,
+        qstep_ratio_list, stats_list, q_index_list, estimated_bitrate_byframe);
+    double bits_diff = fabs(curr_estimate - bit_budget);
+    if (bits_diff <= min_bits_diff) {
+      min_bits_diff = bits_diff;
+      best_q = q;
+    }
+  }
+  return best_q;
+}
+
+TEST(TplModelTest, EstimateFrameRateTest) {
+  GF_GROUP gf_group;
+  init_toy_gf_group(&gf_group);
+
+  VBR_RATECTRL_INFO vbr_rc_info;
+  init_toy_vbr_rc_info(&vbr_rc_info, gf_group.size);
+
+  std::vector<TplTxfmStats> stats_list(gf_group.size);
+  init_toy_tpl_txfm_stats(&stats_list);
+
+  std::vector<double> est_bitrate_list(gf_group.size);
+  init_toy_tpl_txfm_stats(&stats_list);
+  const aom_bit_depth_t bit_depth = AOM_BITS_8;
+
+  const int q = 125;
+
+  // Case1: all scale factors are 0
+  double scale_factors[FRAME_UPDATE_TYPES] = { 0 };
+  double estimate = av1_vbr_rc_info_estimate_gop_bitrate(
+      q, bit_depth, scale_factors, gf_group.size, gf_group.update_type,
+      vbr_rc_info.qstep_ratio_list, stats_list.data(), vbr_rc_info.q_index_list,
+      est_bitrate_list.data());
+  EXPECT_NEAR(estimate, 0, epsilon);
+
+  // Case2: all scale factors are 1
+  for (int i = 0; i < FRAME_UPDATE_TYPES; i++) {
+    scale_factors[i] = 1;
+  }
+  estimate = av1_vbr_rc_info_estimate_gop_bitrate(
+      q, bit_depth, scale_factors, gf_group.size, gf_group.update_type,
+      vbr_rc_info.qstep_ratio_list, stats_list.data(), vbr_rc_info.q_index_list,
+      est_bitrate_list.data());
+  double ref_estimate = 0;
+  for (int i = 0; i < gf_group.size; i++) {
+    ref_estimate += est_bitrate_list[i];
+  }
+  EXPECT_NEAR(estimate, ref_estimate, epsilon);
+
+  // Case3: Key frame scale factor is 0 and others are 1
+  for (int i = 0; i < FRAME_UPDATE_TYPES; i++) {
+    if (i == KF_UPDATE) {
+      scale_factors[i] = 0;
+    } else {
+      scale_factors[i] = 1;
+    }
+  }
+  estimate = av1_vbr_rc_info_estimate_gop_bitrate(
+      q, bit_depth, scale_factors, gf_group.size, gf_group.update_type,
+      vbr_rc_info.qstep_ratio_list, stats_list.data(), vbr_rc_info.q_index_list,
+      est_bitrate_list.data());
+  ref_estimate = 0;
+  for (int i = 0; i < gf_group.size; i++) {
+    if (gf_group.update_type[i] != KF_UPDATE) {
+      ref_estimate += est_bitrate_list[i];
+    }
+  }
+  EXPECT_NEAR(estimate, ref_estimate, epsilon);
+}
+
+TEST(TplModelTest, VbrRcInfoEstimateBaseQTest) {
+  GF_GROUP gf_group;
+  init_toy_gf_group(&gf_group);
+
+  VBR_RATECTRL_INFO vbr_rc_info;
+  init_toy_vbr_rc_info(&vbr_rc_info, gf_group.size);
+
+  std::vector<TplTxfmStats> stats_list(gf_group.size);
+  init_toy_tpl_txfm_stats(&stats_list);
+  const aom_bit_depth_t bit_depth = AOM_BITS_8;
+
+  // Test multiple bit budgets.
+  const std::vector<double> bit_budgets = { 0,     2470,  19200,  30750,
+                                            41315, 65017, DBL_MAX };
+
+  for (double bit_budget : bit_budgets) {
+    // Binary search method to find the optimal q.
+    const int base_q = av1_vbr_rc_info_estimate_base_q(
+        bit_budget, bit_depth, vbr_rc_info.scale_factors, gf_group.size,
+        gf_group.update_type, vbr_rc_info.qstep_ratio_list, stats_list.data(),
+        vbr_rc_info.q_index_list, nullptr);
+    const int ref_base_q = find_gop_q_iterative(
+        bit_budget, bit_depth, vbr_rc_info.scale_factors, gf_group.size,
+        gf_group.update_type, vbr_rc_info.qstep_ratio_list, stats_list.data(),
+        vbr_rc_info.q_index_list, nullptr);
+    if (bit_budget == 0) {
+      EXPECT_EQ(base_q, 255);
+    } else if (bit_budget == DBL_MAX) {
+      EXPECT_EQ(base_q, 0);
+    }
+    EXPECT_EQ(base_q, ref_base_q);
+  }
+}
+#endif  // CONFIG_BITRATE_ACCURACY
 
 }  // namespace
