@@ -8,6 +8,9 @@
  * Media Patent License 1.0 was not distributed with this source code in the
  * PATENTS file, you can obtain it at www.aomedia.org/license/patent.
  */
+#include <memory>
+#include <new>
+
 #include "aom_ports/aom_timer.h"
 #include "test/warp_filter_test_util.h"
 
@@ -124,17 +127,21 @@ void AV1WarpFilterTest::RunSpeedTest(warp_affine_func test_impl) {
   int sub_x, sub_y;
   const int bd = 8;
 
-  uint8_t *input_ = new uint8_t[h * stride];
-  uint8_t *input = input_ + border;
+  std::unique_ptr<uint8_t[]> input_(new (std::nothrow) uint8_t[h * stride]);
+  ASSERT_NE(input_, nullptr);
+  uint8_t *input = input_.get() + border;
 
   // The warp functions always write rows with widths that are multiples of 8.
   // So to avoid a buffer overflow, we may need to pad rows to a multiple of 8.
   int output_n = ((out_w + 7) & ~7) * out_h;
-  uint8_t *output = new uint8_t[output_n];
+  std::unique_ptr<uint8_t[]> output(new (std::nothrow) uint8_t[output_n]);
+  ASSERT_NE(output, nullptr);
   int32_t mat[8];
   int16_t alpha, beta, gamma, delta;
   ConvolveParams conv_params = get_conv_params(0, 0, bd);
-  CONV_BUF_TYPE *dsta = new CONV_BUF_TYPE[output_n];
+  std::unique_ptr<CONV_BUF_TYPE[]> dsta(new (std::nothrow)
+                                            CONV_BUF_TYPE[output_n]);
+  ASSERT_NE(dsta, nullptr);
   generate_warped_model(&rnd_, mat, &alpha, &beta, &gamma, &delta,
                         is_alpha_zero, is_beta_zero, is_gamma_zero,
                         is_delta_zero);
@@ -150,24 +157,21 @@ void AV1WarpFilterTest::RunSpeedTest(warp_affine_func test_impl) {
   sub_y = 0;
   int do_average = 0;
 
-  conv_params = get_conv_params_no_round(do_average, 0, dsta, out_w, 1, bd);
+  conv_params =
+      get_conv_params_no_round(do_average, 0, dsta.get(), out_w, 1, bd);
   conv_params.use_dist_wtd_comp_avg = 0;
 
   const int num_loops = 1000000000 / (out_w + out_h);
   aom_usec_timer timer;
   aom_usec_timer_start(&timer);
   for (int i = 0; i < num_loops; ++i)
-    test_impl(mat, input, w, h, stride, output, 32, 32, out_w, out_h, out_w,
-              sub_x, sub_y, &conv_params, alpha, beta, gamma, delta);
+    test_impl(mat, input, w, h, stride, output.get(), 32, 32, out_w, out_h,
+              out_w, sub_x, sub_y, &conv_params, alpha, beta, gamma, delta);
 
   aom_usec_timer_mark(&timer);
   const int elapsed_time = static_cast<int>(aom_usec_timer_elapsed(&timer));
   printf("warp %3dx%-3d: %7.2f ns\n", out_w, out_h,
          1000.0 * elapsed_time / num_loops);
-
-  delete[] input_;
-  delete[] output;
-  delete[] dsta;
 }
 
 void AV1WarpFilterTest::RunCheckOutput(warp_affine_func test_impl) {
@@ -187,15 +191,22 @@ void AV1WarpFilterTest::RunCheckOutput(warp_affine_func test_impl) {
   // The warp functions always write rows with widths that are multiples of 8.
   // So to avoid a buffer overflow, we may need to pad rows to a multiple of 8.
   int output_n = ((out_w + 7) & ~7) * out_h;
-  uint8_t *input_ = new uint8_t[h * stride];
-  uint8_t *input = input_ + border;
-  uint8_t *output = new uint8_t[output_n];
-  uint8_t *output2 = new uint8_t[output_n];
+  std::unique_ptr<uint8_t[]> input_(new (std::nothrow) uint8_t[h * stride]);
+  ASSERT_NE(input_, nullptr);
+  uint8_t *input = input_.get() + border;
+  std::unique_ptr<uint8_t[]> output(new (std::nothrow) uint8_t[output_n]);
+  ASSERT_NE(output, nullptr);
+  std::unique_ptr<uint8_t[]> output2(new (std::nothrow) uint8_t[output_n]);
+  ASSERT_NE(output2, nullptr);
   int32_t mat[8];
   int16_t alpha, beta, gamma, delta;
   ConvolveParams conv_params = get_conv_params(0, 0, bd);
-  CONV_BUF_TYPE *dsta = new CONV_BUF_TYPE[output_n];
-  CONV_BUF_TYPE *dstb = new CONV_BUF_TYPE[output_n];
+  std::unique_ptr<CONV_BUF_TYPE[]> dsta(new (std::nothrow)
+                                            CONV_BUF_TYPE[output_n]);
+  ASSERT_NE(dsta, nullptr);
+  std::unique_ptr<CONV_BUF_TYPE[]> dstb(new (std::nothrow)
+                                            CONV_BUF_TYPE[output_n]);
+  ASSERT_NE(dstb, nullptr);
   for (int i = 0; i < output_n; ++i) output[i] = output2[i] = rnd_.Rand8();
 
   for (i = 0; i < num_iters; ++i) {
@@ -217,8 +228,8 @@ void AV1WarpFilterTest::RunCheckOutput(warp_affine_func test_impl) {
           for (int jj = 0; jj < 5; ++jj) {
             for (int do_average = 0; do_average <= 1; ++do_average) {
               if (use_no_round) {
-                conv_params =
-                    get_conv_params_no_round(do_average, 0, dsta, out_w, 1, bd);
+                conv_params = get_conv_params_no_round(
+                    do_average, 0, dsta.get(), out_w, 1, bd);
               } else {
                 conv_params = get_conv_params(0, 0, bd);
               }
@@ -229,12 +240,12 @@ void AV1WarpFilterTest::RunCheckOutput(warp_affine_func test_impl) {
                 conv_params.fwd_offset = quant_dist_lookup_table[jj][ii];
                 conv_params.bck_offset = quant_dist_lookup_table[jj][1 - ii];
               }
-              av1_warp_affine_c(mat, input, w, h, stride, output, 32, 32, out_w,
-                                out_h, out_w, sub_x, sub_y, &conv_params, alpha,
-                                beta, gamma, delta);
+              av1_warp_affine_c(mat, input, w, h, stride, output.get(), 32, 32,
+                                out_w, out_h, out_w, sub_x, sub_y, &conv_params,
+                                alpha, beta, gamma, delta);
               if (use_no_round) {
-                conv_params =
-                    get_conv_params_no_round(do_average, 0, dstb, out_w, 1, bd);
+                conv_params = get_conv_params_no_round(
+                    do_average, 0, dstb.get(), out_w, 1, bd);
               }
               if (jj >= 4) {
                 conv_params.use_dist_wtd_comp_avg = 0;
@@ -243,9 +254,9 @@ void AV1WarpFilterTest::RunCheckOutput(warp_affine_func test_impl) {
                 conv_params.fwd_offset = quant_dist_lookup_table[jj][ii];
                 conv_params.bck_offset = quant_dist_lookup_table[jj][1 - ii];
               }
-              test_impl(mat, input, w, h, stride, output2, 32, 32, out_w, out_h,
-                        out_w, sub_x, sub_y, &conv_params, alpha, beta, gamma,
-                        delta);
+              test_impl(mat, input, w, h, stride, output2.get(), 32, 32, out_w,
+                        out_h, out_w, sub_x, sub_y, &conv_params, alpha, beta,
+                        gamma, delta);
               if (use_no_round) {
                 for (j = 0; j < out_w * out_h; ++j)
                   ASSERT_EQ(dsta[j], dstb[j])
@@ -269,11 +280,6 @@ void AV1WarpFilterTest::RunCheckOutput(warp_affine_func test_impl) {
         }
       }
   }
-  delete[] input_;
-  delete[] output;
-  delete[] output2;
-  delete[] dsta;
-  delete[] dstb;
 }
 }  // namespace AV1WarpFilter
 
@@ -320,13 +326,17 @@ void AV1HighbdWarpFilterTest::RunSpeedTest(highbd_warp_affine_func test_impl) {
   // The warp functions always write rows with widths that are multiples of 8.
   // So to avoid a buffer overflow, we may need to pad rows to a multiple of 8.
   int output_n = ((out_w + 7) & ~7) * out_h;
-  uint16_t *input_ = new uint16_t[h * stride];
-  uint16_t *input = input_ + border;
-  uint16_t *output = new uint16_t[output_n];
+  std::unique_ptr<uint16_t[]> input_(new (std::nothrow) uint16_t[h * stride]);
+  ASSERT_NE(input_, nullptr);
+  uint16_t *input = input_.get() + border;
+  std::unique_ptr<uint16_t[]> output(new (std::nothrow) uint16_t[output_n]);
+  ASSERT_NE(output, nullptr);
   int32_t mat[8];
   int16_t alpha, beta, gamma, delta;
   ConvolveParams conv_params = get_conv_params(0, 0, bd);
-  CONV_BUF_TYPE *dsta = new CONV_BUF_TYPE[output_n];
+  std::unique_ptr<CONV_BUF_TYPE[]> dsta(new (std::nothrow)
+                                            CONV_BUF_TYPE[output_n]);
+  ASSERT_NE(dsta, nullptr);
 
   generate_warped_model(&rnd_, mat, &alpha, &beta, &gamma, &delta,
                         is_alpha_zero, is_beta_zero, is_gamma_zero,
@@ -345,24 +355,21 @@ void AV1HighbdWarpFilterTest::RunSpeedTest(highbd_warp_affine_func test_impl) {
   sub_y = 0;
   int do_average = 0;
   conv_params.use_dist_wtd_comp_avg = 0;
-  conv_params = get_conv_params_no_round(do_average, 0, dsta, out_w, 1, bd);
+  conv_params =
+      get_conv_params_no_round(do_average, 0, dsta.get(), out_w, 1, bd);
 
   const int num_loops = 1000000000 / (out_w + out_h);
   aom_usec_timer timer;
   aom_usec_timer_start(&timer);
 
   for (int i = 0; i < num_loops; ++i)
-    test_impl(mat, input, w, h, stride, output, 32, 32, out_w, out_h, out_w,
-              sub_x, sub_y, bd, &conv_params, alpha, beta, gamma, delta);
+    test_impl(mat, input, w, h, stride, output.get(), 32, 32, out_w, out_h,
+              out_w, sub_x, sub_y, bd, &conv_params, alpha, beta, gamma, delta);
 
   aom_usec_timer_mark(&timer);
   const int elapsed_time = static_cast<int>(aom_usec_timer_elapsed(&timer));
   printf("highbd warp %3dx%-3d: %7.2f ns\n", out_w, out_h,
          1000.0 * elapsed_time / num_loops);
-
-  delete[] input_;
-  delete[] output;
-  delete[] dsta;
 }
 
 void AV1HighbdWarpFilterTest::RunCheckOutput(
@@ -384,15 +391,22 @@ void AV1HighbdWarpFilterTest::RunCheckOutput(
   // The warp functions always write rows with widths that are multiples of 8.
   // So to avoid a buffer overflow, we may need to pad rows to a multiple of 8.
   int output_n = ((out_w + 7) & ~7) * out_h;
-  uint16_t *input_ = new uint16_t[h * stride];
-  uint16_t *input = input_ + border;
-  uint16_t *output = new uint16_t[output_n];
-  uint16_t *output2 = new uint16_t[output_n];
+  std::unique_ptr<uint16_t[]> input_(new (std::nothrow) uint16_t[h * stride]);
+  ASSERT_NE(input_, nullptr);
+  uint16_t *input = input_.get() + border;
+  std::unique_ptr<uint16_t[]> output(new (std::nothrow) uint16_t[output_n]);
+  ASSERT_NE(output, nullptr);
+  std::unique_ptr<uint16_t[]> output2(new (std::nothrow) uint16_t[output_n]);
+  ASSERT_NE(output2, nullptr);
   int32_t mat[8];
   int16_t alpha, beta, gamma, delta;
   ConvolveParams conv_params = get_conv_params(0, 0, bd);
-  CONV_BUF_TYPE *dsta = new CONV_BUF_TYPE[output_n];
-  CONV_BUF_TYPE *dstb = new CONV_BUF_TYPE[output_n];
+  std::unique_ptr<CONV_BUF_TYPE[]> dsta(new (std::nothrow)
+                                            CONV_BUF_TYPE[output_n]);
+  ASSERT_NE(dsta, nullptr);
+  std::unique_ptr<CONV_BUF_TYPE[]> dstb(new (std::nothrow)
+                                            CONV_BUF_TYPE[output_n]);
+  ASSERT_NE(dstb, nullptr);
   for (int i = 0; i < output_n; ++i) output[i] = output2[i] = rnd_.Rand16();
 
   for (i = 0; i < num_iters; ++i) {
@@ -415,8 +429,8 @@ void AV1HighbdWarpFilterTest::RunCheckOutput(
           for (int jj = 0; jj < 5; ++jj) {
             for (int do_average = 0; do_average <= 1; ++do_average) {
               if (use_no_round) {
-                conv_params =
-                    get_conv_params_no_round(do_average, 0, dsta, out_w, 1, bd);
+                conv_params = get_conv_params_no_round(
+                    do_average, 0, dsta.get(), out_w, 1, bd);
               } else {
                 conv_params = get_conv_params(0, 0, bd);
               }
@@ -428,14 +442,15 @@ void AV1HighbdWarpFilterTest::RunCheckOutput(
                 conv_params.bck_offset = quant_dist_lookup_table[jj][1 - ii];
               }
 
-              av1_highbd_warp_affine_c(mat, input, w, h, stride, output, 32, 32,
-                                       out_w, out_h, out_w, sub_x, sub_y, bd,
-                                       &conv_params, alpha, beta, gamma, delta);
+              av1_highbd_warp_affine_c(mat, input, w, h, stride, output.get(),
+                                       32, 32, out_w, out_h, out_w, sub_x,
+                                       sub_y, bd, &conv_params, alpha, beta,
+                                       gamma, delta);
               if (use_no_round) {
                 // TODO(angiebird): Change this to test_impl once we have SIMD
                 // implementation
-                conv_params =
-                    get_conv_params_no_round(do_average, 0, dstb, out_w, 1, bd);
+                conv_params = get_conv_params_no_round(
+                    do_average, 0, dstb.get(), out_w, 1, bd);
               }
               if (jj >= 4) {
                 conv_params.use_dist_wtd_comp_avg = 0;
@@ -444,9 +459,9 @@ void AV1HighbdWarpFilterTest::RunCheckOutput(
                 conv_params.fwd_offset = quant_dist_lookup_table[jj][ii];
                 conv_params.bck_offset = quant_dist_lookup_table[jj][1 - ii];
               }
-              test_impl(mat, input, w, h, stride, output2, 32, 32, out_w, out_h,
-                        out_w, sub_x, sub_y, bd, &conv_params, alpha, beta,
-                        gamma, delta);
+              test_impl(mat, input, w, h, stride, output2.get(), 32, 32, out_w,
+                        out_h, out_w, sub_x, sub_y, bd, &conv_params, alpha,
+                        beta, gamma, delta);
 
               if (use_no_round) {
                 for (j = 0; j < out_w * out_h; ++j)
@@ -471,12 +486,6 @@ void AV1HighbdWarpFilterTest::RunCheckOutput(
         }
       }
   }
-
-  delete[] input_;
-  delete[] output;
-  delete[] output2;
-  delete[] dsta;
-  delete[] dstb;
 }
 }  // namespace AV1HighbdWarpFilter
 #endif  // CONFIG_AV1_HIGHBITDEPTH
